/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.ui.data.Store");

dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojo.data.ItemFileWriteStore");
dojo.require("com.ibm.rational.cm.web._base");
dojo.require("com.ibm.rational.cm.web.ui.page.CqLoginController");

(function(){

var cq = com.ibm.rational.cm.web;
var loginController = com.ibm.rational.cm.cqLoginController;

dojo.declare(
	"com.ibm.rational.cm.web.ui.data.StoreMixin",
	null,
{
	// Handler holds onto an xhr request handler and is used for Cancel actions
	_handler: null,
	
	_overrideXhrHandler: function(/* String */ funcName, args){
		var self = this;
		loginController.process({
			callback: function(){
				// keep the original handler
				var origHandler = dojo.xhrGet;

				// override handler
				dojo.xhrGet = function(args1){
					// support additional 'content' argument to pass to XHR
					if(args.length > 0){
						args1.content = args[0].content;
					}
					args1.handleError = true; // custom argument
					args1.preventCache = true;
					if(self.requestMethod == "POST"){
						self._handler = cq.xhrPost(args1);
					}else{
						// pass 'origHandler' to avoid infinite loop of dojo.xhrGet -> cq.xhrGet -> dojo.xhrGet
						self._handler = cq.xhrGet(args1, origHandler);
					}
					return self._handler;
				}
		
				// run function with overriden handler
				self.inherited(funcName, args);

				// restore the original handler
				dojo.xhrGet = origHandler;		
			}			
		});
	},
	
	// Override dojo.data.ItemFileReadStore
	// so that xhrPost can be used
	_fetchItems: function(	/* Object */ keywordArgs, 
							/* Function */ findCallback, 
							/* Function */ errorCallback){
		this._overrideXhrHandler("_fetchItems", arguments);
	},

	// Override dojo.data.ItemFileReadStore
	// so that xhrPost can be used
	fetchItemByIdentity: function(/* Object */ keywordArgs){
		this._overrideXhrHandler("fetchItemByIdentity", arguments);
	},

	// Override dojo.data.ItemFileReadStore
	// so that xhrPost can be used
	_forceLoad: function(){
		this._overrideXhrHandler("_forceLoad", arguments);
	},
	
	// user overridable function
	// formats the data to dojo.data format
	// by default, just pass the data as it is in the original format
	// if null is returned, it doesn't call superclass's _getItemsFromLoadedData
	format: function(dataObject){
		return dataObject;
	},
	
	// Cancel current fetch operation
	cancelRequest: function(){
		if (this._handler !== null)
		{
			this._handler.cancel();
			this._handler = null;
		}
	},
	
	// override to notify completion of store.fetch
	// since this dojo.connect can't be used for such purpose
	_getItemsFromLoadedData: function(/* Object */ dataObject){
		dataObject = this.format(dataObject);
		if(dataObject){
			var args = arguments;
			if(args.length > 0){
				args[0] = dataObject;
			}
			this.inherited(args);
		}
	}
});

dojo.declare(
	"com.ibm.rational.cm.web.ui.data.ReadStore",
	[ dojo.data.ItemFileReadStore, com.ibm.rational.cm.web.ui.data.StoreMixin ],
{
	constructor: function(/* Object */ keywordParameters){
		this.requestMethod = keywordParameters.requestMethod;
	}
});

dojo.declare(
	"com.ibm.rational.cm.web.ui.data.WriteStore",
	[ dojo.data.ItemFileWriteStore, com.ibm.rational.cm.web.ui.data.StoreMixin ],
{
	constructor: function(/* Object */ keywordParameters){
		this.requestMethod = keywordParameters.requestMethod;
	}
});

})();