 /*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.viewlets.web.ui.internal.QueryClient2");

dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("jazz.client.xhr");

(function() {

var QueryClient = com.ibm.team.workitem.web.client.internal.QueryClient;

/*
 * A temporary adapter for the work item QueryClient to skip oauth when running
 * C/ALM queries. This was added as an adapter rather than in the actual client to
 * avoid destabilizing work items during 2.0 RC3, and should be included in the
 * QueryClient eventually.
 * 
 * See:
 * 85357: Support skipOAuth in work item QueryClient
 * 83541: IOAuthConsumerService: allow to skip auth popup
 */
com.ibm.team.workitem.viewlets.web.ui.internal.QueryClient2 = {
	
	/*
	 * See QueryClient#getQuery2. Fetching queries does not trigger oauth (only
	 * running them).
	 */
	getQuery2: function(serviceResponseHandler, args) {
		return QueryClient.getQuery2(serviceResponseHandler, args);
	},
	
	/*
	 * See QueryClient#getHistogram. Accepts an additional "skipOAuth" property in
	 * the userArgs, which determines whether or not the oauth interceptor will be
	 * skipped.
	 * 
	 * The XHR object is returned as an "xhr" property on the service response handler
	 * if oauth is skipped, so that we can read the oauth HTTP header to get the URL.
	 */
	getHistogram: function(serviceResponseHandler, userArgs) {
		return this._invoke(QueryClient.getHistogram, arguments, userArgs.skipOAuth, serviceResponseHandler);
	},	
	
	/*
	 * See QueryClient#getResultSet. Accepts an additional "skipOAuth" property in
	 * the userArgs, which determines whether or not the oauth interceptor will be
	 * skipped.
	 * 
	 * The XHR object is returned as an "xhr" property on the service response handler
	 * if oauth is skipped, so that we can read the oauth HTTP header to get the URL.
	 */
	getResultSet: function(query, userArgs, serviceResponseHandler){
		return this._invoke(QueryClient.getResultSet, arguments, userArgs.skipOAuth, serviceResponseHandler);
	},
	
	/*
	 * Invoke the request with skipOAuth support.
	 */
	_invoke: function(/* function */ func, /* array */ args, /* boolean */ skipOAuth, /* ServiceResponseHandler */ srh) {
		var xhr = jazz.client.xhr;
		try {
			if (skipOAuth) {
				// temporarily hijack jazz.client.xhr for request
				jazz.client.xhr = function(method, args){
					args.skipInterceptors = ["jazz.oauth"];
					var deferred = xhr.apply(this, arguments);
					// attach xhr so we can read response headers
					srh.xhr = deferred.ioArgs.xhr;
					return deferred;
				};
			}
			return func.apply(QueryClient, args);
		}
		finally {
			if (skipOAuth) {
				// reset jazz.client.xhr back
				jazz.client.xhr = xhr;
			}
		}
	}
};

})();
