/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportDescriptorSelectionDialog");

dojo.require("dijit.Tree");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojo.i18n");

dojo.require("jazz.ui.Dialog");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() {
	
dojo.declare("com.ibm.team.reports.web.ui.internal.ReportDescriptorSelectionDialog", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ReportDescriptorSelectionDialog.html"),
	widgetsInTemplate: true,
	
	constructor: function (args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
		if (args == null) {
			args = {};
		}
		this.width = args.width || "450px";
		var properties = {
			"dialogTitle" : this.messages.selectReportTemplate,
			"width" : "450px",
			"closable": true,
			"reportTemplates": []
		};
		for (var prop in properties) {
			if (args[prop] != null) {
				this[prop] = args[prop];
			}
			else {
				this[prop] = properties[prop];
			}
		}
	},
	
	postCreate: function () {
		this._addTemplates(this.reportTemplates);
		this.dialog = new jazz.ui.Dialog({
			contentNode: this.contentNode,
			primaryTitle: this.dialogTitle,
			width: this.width,
			closable: this.closable
		});
		this.connect(this.okButton, "onclick", "_handleOk");
		this.connect(this.cancelButton, "onclick", "_handleCancel");
		this.connect(this.userSelector,"ondblclick", "_handleDoubleClick");		
	},
	
	_handleOk: function (e) {
		dojo.stopEvent(e);
		this.dialog.close();
		this.onOk(this._currentTemplate);
	},
	
	_handleCancel: function (e) {
		dojo.stopEvent(e);
		this.selectedReport = null;
		this.dialog.close();
		this.onCancel();
	},
	
	_handleDoubleClick: function (e) {
		this._handleOk(e);
	},
	
	onOk: function(reportTemplate) {
	},
	
	onCancel: function() {
	},
	
	_addTemplates: function(args) {
		if (args == null) args = [];
		// First sort them alphabetically
		function sortFunction(a, b) {
			return a.name < b.name ? -1 : (a.name > b.name ? 1 : 0);
		}
		var sorted = args.sort(sortFunction);
		this._templates = sorted;
		var store = this._createTemplateDataStore(args);
		this._createTemplateTree(store);			
	},
	
	_createTemplateDataStore: function(templates) {
		var store = new Object();
		store.label = "name";
		store.identifier = "reportUUID";
		var items = new Array();
		store.items = templates;
		return store;
	},
	
	_createTemplateTree: function(dataStore) {
		var treeStore = new dojo.data.ItemFileReadStore({
			data: dataStore
		});
		var tree = new dijit.Tree({
			store: treeStore,
			widgetId: "selectTemplateTreeWidget",
			getIconClass: function(item) {
				return "iconTemplate";
			}
		});

		this._templateTreeWidget = tree;
		this._templateTree.appendChild(tree.domNode);
		
		this.connect(tree, "focusNode", this._onFocusTemplate);
	},
	
	_onFocusTemplate: function(node) {
		// Get the old focus node
		var oldFocusNode = this._currentTemplateNode;
	
		// Remove selection from old focus node
		if ((node != oldFocusNode) &&
			(oldFocusNode != null)){
			dojo.removeClass(oldFocusNode.contentNode, "tree-focus-entry");
		}
		// Add selection to new focus node
		if (node != null) {
			dojo.addClass(node.contentNode, "tree-focus-entry");
		}
		
		if (node) {
			this._currentTemplate = node.item;
			this._currentTemplateNode = node;
		} else {
			this._currentTemplate = null;
			this._currentTemplateNode = null;
		}
		
		this._updateEnablement();
	},
	
	_updateEnablement: function() {
		if (this._currentTemplate == null) {
			this.okButton.disabled = true;
		} else {
			this.okButton.disabled = false;
		}
	}
});

})();