/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportParametersPanel");

dojo.require("dojo.i18n");

dojo.require("jazz.ui.styledBox.TrimStyles");
dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.ui.Dialog");

dojo.require("com.ibm.team.reports.web.ui.internal.ReportParam");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() {
	
	var TrimStyles = jazz.ui.styledBox.TrimStyles;
	var Dialog = jazz.ui.Dialog;		

    dojo.declare("com.ibm.team.reports.web.ui.internal.ReportParametersPanel", [dijit._Widget, dijit._Templated], {
		
        templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ReportParametersPanel.html"),
		padding: "10px",
		
		constructor: function(props) {
			dojo.mixin(this, props);
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
			// Flag to determine whether or not to remove the 'loading...' text, should only be done once,
			// prior to the first parameter widget being appended.
			this._appendedParameter = false;
			this.dirtyListener = null;
		},
		
		postCreate: function() {
			this._rootDiv.style.padding = this.padding;
		},
		
		setDirtyListener: function(object) {
			this.dirtyListener = object;
		},
		
		showAsDialog: function(onCloseFunc) {
			var self = this;
			// Create the dialog
			this._dialog = new Dialog({
				contentNode: this.domNode,
				primaryTitle: this.messages.parameters,
				width: "925px",
				isClosable: true,
				onClose: function() {
					if (onCloseFunc) {
						onCloseFunc();	
					}
				}
			});
			if (this._focusOnOpenElem && typeof this._focusOnOpenElem.focus == 'function') {
				this._focusOnOpenElem.focus();
			}
		},
		
		focusOnOpen: function(elem) {
			this._focusOnOpenElem = elem;
		},
		
		appendParamGroup: function(reportParamGroup) {
			if (!this._appendedParameter) {
				this._removeChildren(this._mainSectionLeft);
				this._appendedParameter = true;	
			}
			if (this.dirtyListener != null) {
				reportParamGroup.setDirtyListener(this.dirtyListener);
			}
			this._mainSectionLeft.appendChild(reportParamGroup.domNode);
		},
		
		appendParam: function(reportParam) {
			if (!this._appendedParameter) {
				this._removeChildren(this._mainSectionLeft);
				this._appendedParameter = true;	
			}
			if (this.dirtyListener != null) {
				reportParam.setDirtyListener(this.dirtyListener);
			}
			var styledBox = new jazz.ui.StyledBox({
				closable: false, 
				primaryTitle: reportParam.getDisplayLabel() + (reportParam.isRequired() ? "*" : ""), 
				contentNode: reportParam.domNode,
				"class": "box-single",
				trim: TrimStyles.BLUE_ALPHA_SHADOW,
				headerDefaultToHoverState: true
			});
			this._mainSectionLeft.appendChild(styledBox.domNode);
		},
		
		setMainSectionRight: function(domNode) {
			this._hasRightSection = true;
			this._mainSectionRight.appendChild(domNode);
		},
		
		/*
	 	 * @param Boolean show
	 	 * 
	 	 * @return void
	 	 */			
		showPreviewPane: function(show) {
			
			if (show) {
				// Parameters Pane
				dojo.removeClass(this._mainSectionLeft, "parametersPaneLayoutOff");
				dojo.addClass(this._mainSectionLeft, "parametersPaneLayoutOn");
				// Preview Pane
				dojo.removeClass(this._previewPaneLayout, "previewPaneLayoutOff");
				dojo.addClass(this._previewPaneLayout, "previewPaneLayoutOn");			
			} else {
				// Parameters Pane
				dojo.removeClass(this._mainSectionLeft, "parametersPaneLayoutOn");
				dojo.addClass(this._mainSectionLeft, "parametersPaneLayoutOff");
				// Preview Pane
				dojo.removeClass(this._previewPaneLayout, "previewPaneLayoutOn");			
				dojo.addClass(this._previewPaneLayout, "previewPaneLayoutOff");
			}
		},
		
		setBottomButtonsSection: function(domNode) {
			this._bottomSection.appendChild(domNode);
		},
		
		setTopInputFieldSection: function(domNode) {
			this._topSection.appendChild(domNode);
		},
		
		closeDialog: function() {
			if (this._dialog) {
				this._dialog.close();
				delete this._dialog;
			}
		},
		
		_removeChildren: function(element){
            while (element.firstChild) {
                element.removeChild(element.firstChild);
            }
        }
		
	});
	
})();
