/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.EditorErrorHandler"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.EditorErrorHandler", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/EditorErrorHandler.html"), //$NON-NLS-1$ //$NON-NLS-2$
		constructor: function(arg){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function(){
			if (this.headerMessage) {
				this.headerMessageDiv.appendChild(document.createTextNode(this.headerMessage));
			}
			if (this.message) {
				this.exceptionMessageDiv.appendChild(document.createElement(ProcessConstants.BR));
				this.exceptionMessageDiv.appendChild(document.createTextNode(this.message));
			}
			if (this.isErrorDataOfInterest(this.errorData)) {
				var msg = this.errorData.internalReportInfos[0].data;
				var stackTraceLines = msg.split("\n"); //$NON-NLS-1$
				if (stackTraceLines.length > 0) {
					for (var i = 0; i < stackTraceLines.length; i++) {
						var span;
						if (stackTraceLines[i].indexOf("\t") !== 0) { //$NON-NLS-1$
							// No left pad for the Caused By lines
							span = this.createStackTraceLineSpan(stackTraceLines[i], "0"); //$NON-NLS-1$
						} else {
							span = this.createStackTraceLineSpan(stackTraceLines[i]);
						}
						this.errorClassMessageDiv.appendChild(span);
					}
					
					this.showDetailsLink.style.display = ProcessConstants.BLOCK;
				}
			} else if (this.exception && this.exception.length > 0 && !this.shouldIgnoreException()) {
				this.errorClassMessageDiv.appendChild(document.createTextNode(this.errorClass + ":")); //$NON-NLS-1$
				for (i = 0; i < this.exception.length; i++) {
					span = this.createStackTraceLineSpan(dojo.string.substitute(this.messages.EditorErrorHandler_At, [this.exception[i]]));
					this.errorClassMessageDiv.appendChild(span);
				}
				this.showDetailsLink.style.display = ProcessConstants.BLOCK;
			}
			
			this.connect(this.showDetailsLink, "onclick", function(e){ //$NON-NLS-1$
				dojo.stopEvent(e);
				this.showDetailsLink.style.display = ProcessConstants.NONE;
				this.hideDetailsLink.style.display = ProcessConstants.BLOCK;
				this.exceptionDiv.style.display = ProcessConstants.BLOCK;
			});
			
			this.connect(this.hideDetailsLink, "onclick", function(e){ //$NON-NLS-1$
				dojo.stopEvent(e);
				this.showDetailsLink.style.display = ProcessConstants.BLOCK;
				this.hideDetailsLink.style.display = ProcessConstants.NONE;
				this.exceptionDiv.style.display = ProcessConstants.NONE;
			});
			
		},
		
		createStackTraceLineSpan: function(stackTraceLine, leftPad){
			if (leftPad === undefined) {
				leftPad = "2em"; //$NON-NLS-1$
			}
			var span = document.createElement("span"); //$NON-NLS-1$
			dojo.style(span, "marginLeft", leftPad); //$NON-NLS-1$
			dojo.style(span, "display", ProcessConstants.BLOCK); //$NON-NLS-1$
			span.appendChild(document.createTextNode(stackTraceLine));
			return span;
			
		},
		
		isErrorDataOfInterest: function(errorData){
			return errorData !== undefined &&
			(errorData.identifier === "com.ibm.team.process.server.initializeProjectArea" || errorData.identifier === "com.ibm.team.process.projectLinks" || errorData.identifier === "com.ibm.team.process.newMembersWithoutRole") && //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			errorData.internalReportInfos !== undefined &&
			errorData.internalReportInfos.length > 0 &&
			(errorData.internalReportInfos[0].identifier === "com.ibm.team.process.internal.operationException" || errorData.internalReportInfos[0].identifier === "com.ibm.team.process.internal.linksException" || errorData.internalReportInfos[0].identifier === "com.ibm.team.process.internal.newMembersWithoutRoles") && //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			errorData.internalReportInfos[0].data !== undefined;
		},
		
		shouldIgnoreException: function(){
			if (this.exceptionClassesToIgnore) {
				for (var i = 0; i < this.exceptionClassesToIgnore.length; i++) {
					if (this.exceptionClassesToIgnore[i] === this.errorClass) {
						return true;
					}
				}
			}
			return false;
		}
		
	});
	
})();
