﻿/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

import flash.external.ExternalInterface;

 
class ChartEngine {

	private static var _INSTANCE:ChartEngine = null;
	
	private static var _MIN_STAGE_WIDTH:Number = 120; // Pixels

	private static var _MIN_STAGE_HEIGHT:Number = 60; // Pixels
	
	private var _model:ChartModel;
	
	private var _chartArea:ChartArea;

	private var _errorArea:ErrorArea;
	
	private var _chartParameters:String;
	
	private var _resizeEventHandler:Object;
	
	private var _preserveModel:Boolean;
	
	private var _mouseMoveListeners:Object;
	
	
	private function ChartEngine() {
		// NO-OP
	}	

	public static function getInstance():ChartEngine {
		if (ChartEngine._INSTANCE == null) {
			ChartEngine._INSTANCE = new ChartEngine();
		}
		return ChartEngine._INSTANCE;
	}	
	
	public function run():Void {
		// Setup stage
		// No scaling
		Stage.scaleMode = "noScale";
		// Top left alignment
		Stage.align = "TL";
		
		// TEST
		//DebugArea.alert(String(Stage.width));
		//DebugArea.alert(String(Stage.height));
		
		// IE Workaround
		// Timing problem peculiar to the ActiveX player. 
		// Use an onEnterFrame handler to wait until the stage size is set by the player.
		var context = this;
		
		_root.onEnterFrame = function() {
			if (Stage.width > 0) {
				delete _root.onEnterFrame;
				context._execute();
			}
		}	
		
	}
	
	private function _execute():Void {
		this._renderChart();
		this._connectRefreshEventHandler();
		
		if (this._model.isDebug) {
			this._chartArea.getDebugArea().debug("EXECUTE");
		}		
	}
	
	private function _renderChart():Void {
		
		// Validate stage
		if (this._isStageValid() == false) {
			this._errorArea = new ErrorArea();
			this._errorArea.draw(_root);
			return;		
		}
		
		var timer = new Timer("TIME");		
		timer.start();

		// Load model
		var status:Boolean = this._loadModel();
		
		if (status == false) {
			this._errorArea = new ErrorArea();
			this._errorArea.draw(_root);
			return;		
		}
		
		// TEST
		//Tester.dumpModel(this._model);
		
		// Create chart area
		this._chartArea = new ChartArea();
		this._chartArea.draw(_root);
		
		// Connect resize event handler
		this._connectResizeEventHandler();		
		// Connect mouse move event handler
		this._connectMouseMoveEventHandler();
		
		timer.stop();

		if (this._model.isDebug) {
			this._chartArea.getDebugArea().debug(timer.getElapsedTimeString());
		}
		
	}
	
	private function _loadModel():Boolean {
		// TEST
		//Tester.loadParameters8();
		
		if (this._model) {
			// Resize event
			// Reuse existing model
			return true;
		} else if (this._chartParameters) {
			// Refresh called
			// Load model from String
			this._model = new ChartModel();
			this._model.loadParametersFromString(this._chartParameters);
			return this._model.isValid();
		} else if (_root._fpresentation) {
			// Flash object created
			// Load model from Root
			this._model = new ChartModel();
			this._model.loadParametersFromRoot();
			return this._model.isValid();
		}
		
		return false;
	}
	
	private function _isStageValid():Boolean {
		// Note: Flash movie clips have a width / height limit of 2880 pixels
		// http://www.flashuser.net/general/flash-limitations.html
		// TODO: MP: Enforce maximum stage dimensions
		if (Stage.width < ChartEngine._MIN_STAGE_WIDTH) {
			return false;
		} else if (Stage.height < ChartEngine._MIN_STAGE_HEIGHT) {
			return false;
		}
		return true;
	}
	
	public function refresh(chartParameters:String):Void {
		// TEST
		//if (this._model.isDebug) {
		//	this._chartArea.getDebugArea().debug(chartParameters);
		//}
		this.destroy();
		this._chartParameters = chartParameters;
		this._renderChart();

		if (this._model.isDebug) {
			this._chartArea.getDebugArea().debug("REFRESH");
		}
	}
	
	public function resize():Void {
		
		if (this._isStageValid() == false) {
			// Ignore invalid stage resize events
			// e.g. IE browser triggers 0 width X 0 height stage resize event when 
			// Flash object is taken off DOM.
			return;
		}
		
		this._preserveModel = true;
		this._model.animation = false;
		
		this.destroy();
		this._renderChart();
		
		if (this._model.isDebug) {
			this._chartArea.getDebugArea().debug("RESIZE: " + Stage.width + " X " + Stage.height);
		}
	}
	
	public function destroy():Void {
		// Model
		if (!this._preserveModel && this._model) {
			this._model.destroy();
			delete this._model;
		}
		// Chart area
		if (this._chartArea) {
			this._chartArea.destroy();
			delete this._chartArea;
		}	
		// Error area
		if (this._errorArea) {
			this._errorArea.destroy();
			delete this._errorArea;
		}
		// Chart parameters
		delete this._chartParameters;
		// Preserve model
		delete this._preserveModel;
		// Resize event handler		
		this._disconnectResizeEventHandler();
		// Mouse move event handler
		this._disconnectMouseMoveEventHandler();
	}
	
	private function _connectRefreshEventHandler():Void {
		var methodName:String = "refresh";
		var instance:Object = this;
		var method:Function = refresh;		
		var status:Boolean = ExternalInterface.addCallback(methodName, instance, method);		
		
		if (!status && this._model.isDebug) {
			this._chartArea.getDebugArea().debug("ERROR: Failed to connect refresh event handler");
		}
	}
	
	public function getModel():ChartModel {
		return this._model;
	}
	
	public function getChartArea():ChartArea {
		return this._chartArea;
	}
	
	public function getChartParameters():String {
		return this._chartParameters;
	}	
	
	public function setChartParameters(chartParameters:String):Void {
		this._chartParameters = chartParameters;
	}		
	
	private function _connectResizeEventHandler():Void {
		
		if (this._resizeEventHandler) {
			return;
		}

		var context = this;
		this._resizeEventHandler = new Object();

		this._resizeEventHandler.onResize = function () {
			context.resize();
		}			

		Stage.addListener(this._resizeEventHandler);	
	}
	
	private function _disconnectResizeEventHandler():Void {
		if (this._resizeEventHandler) {
			Stage.removeListener(this._resizeEventHandler);
			delete this._resizeEventHandler;
		}
	}	
	
	private function _connectMouseMoveEventHandler():Void {

		this._mouseMoveListeners = new Object();
		var context:ChartEngine = this;
		
		_root.onMouseMove = function() {
			for (var id:String in context._mouseMoveListeners) { 
				var listener:IMouseMoveEventListener = context._mouseMoveListeners[id];
				listener.mouseMoved(_root._xmouse, _root._ymouse);
			}
		};				
	}
	
	private function _disconnectMouseMoveEventHandler():Void {
		if (_root.onMouseMove) {
			delete _root.onMouseMove;
		}
		delete this._mouseMoveListeners;
	}
	
	public function addMouseMoveListener(id:String, listener:IMouseMoveEventListener) {
		this._mouseMoveListeners[id] = listener;
	}
	
	public function removeMouseMoveListener(id:String) {
		if (this._mouseMoveListeners[id]) {
			delete this._mouseMoveListeners[id];
		}
	}	
	
}