/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.query.CellElementViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

(function() {
	
/*
 * Summary: A cell widget's parent class representing a "td" element in tree-table style rendering. This class provides:
 * 			1) registering click event handler and will transfer the handling to the row widget which constructed by several cell widgets.
 * 			2) interface to get the cell data.
 * 			3) interface to get the row widget container of the cell widget, for keeping the "tr-has-tds" relations in table.
 */


dojo.declare("com.ibm.team.build.web.ui.internal.query.CellElementViewer",
 [dijit._Widget, dijit._Templated],
{     
	templateString: "<td><div class='cellWrapper' dojoAttachPoint='content'></div></td>",
	rowContainer: null,
	cellType: "",
	constructor: function(cellArgs) {
		this.rowContainer = cellArgs.rowContainer;
		this.cellType = cellArgs.cellType;
	},
	
	postCreate: function() {
		this.content.innerHTML = this.getCellData(this.cellType);
		var iconClass = this.getCellIconClass(this.cellType);
		if (iconClass)
			this.addIcon(iconClass);
		this.content.style.paddingLeft = this.getIndent();
		this.registerClickHandler();		
	},
	
	registerClickHandler: function(){
		dojo.connect(this.domNode, "onclick", this, "handleClick" );
	},
	
	getCellContainer: function(){
		return this.domNode;
	},
	
	// Will be overrided by sub widget class which provides specific data for cell element to render.
	getCellData: function(cellType){
		return "";
	},
	
	getCellIconClass: function(cellType) {
		return "";
	},
	
	getIndent: function(){
		var indentCounts = this.rowContainer.getTreeLevel() + 1.5;
		return indentCounts + "em"
	},
	
	// Transfer the click event handling to the row widget which including this cell widget.
	handleClick: function(){
		this.rowContainer.handleFirstColumnClick();
	},
	
	addTwistie: function(){
		this._twistie = document.createElement("div");
		this._twistie.className = "twistie twistie-closed";
		this.content.style.marginLeft = "-11px";
		this.content.insertBefore(this._twistie, this.content.firstChild);
		return this._twistie;
	},
	addIcon: function(className) {
		var icon = document.createElement("div");
		icon.className = "cell-icon " + className;
		this.content.insertBefore(icon, this.content.firstChild);
	},
	destroy: function() {
		this.inherited(arguments);
	}
	
});
	
})();
