/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.query.CompilationCellElementViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.build.web.ui.internal.Constants");
dojo.require("com.ibm.team.build.web.ui.internal.query.CellElementViewer");
dojo.require("com.ibm.team.build.web.ui.internal.utils.ResultUtil");

dojo.require("jazz.util.html");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

(function() {

/*
 * Summary: Inherit from the CellElementViewer cell widget. This class provides:
 * 			1) provides the specfic content to cell in tree-table's rows of build result's Compilation tab.
 */

var Constants = com.ibm.team.build.web.ui.internal.Constants;
var CellElementViewer = com.ibm.team.build.web.ui.internal.query.CellElementViewer;
var ResultUtil = com.ibm.team.build.web.ui.internal.utils.ResultUtil;
var messages = dojo.i18n.getLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

dojo.declare("com.ibm.team.build.web.ui.internal.query.CompilationCellElementViewer",
 [CellElementViewer],
{     			
	getCellData: function(cellType){
		var rowData = this.rowContainer.getRowData();
		var escapedName;
		if(cellType == Constants.COMPILE_ERRORS_COLUMN){
			dojo.addClass(this.domNode, "numbersColumn");
			return rowData.errorCount || 0;		
		}
		if(cellType == Constants.COMPILE_WARNINGS_COLUMN){
			dojo.addClass(this.domNode, "numbersColumn");
			return rowData.warningCount || 0;		
		}
		if(cellType == Constants.COMPILE_COMPONENT_NAME_COLUMN){
			return rowData.name ? jazz.util.html.escape(rowData.name) : "<span class='none-text'>" + messages.noComponent + "</span>";			
		}
		if(cellType == Constants.COMPILE_PACKAGE_NAME_COLUMN){
			escapedName = rowData.packageName ? jazz.util.html.escape(rowData.packageName) : "";
			return escapedName ? ResultUtil.insertWordBreaks(escapedName, [".","_"]): "";
		}
		if(cellType == Constants.COMPILE_FILE_NAME_COLUMN){
			var name = rowData.sourceName || rowData.label; // sourceName for java result, label for simple contribution
			escapedName = name ? jazz.util.html.escape(name) : "";
			return escapedName ? ResultUtil.insertWordBreaks(escapedName, [".","_"]): "";
		}
	},
	
	getCellIconClass: function(cellType) {
		var rowData = this.rowContainer.getRowData();
		if(cellType == Constants.COMPILE_COMPONENT_NAME_COLUMN){
			return rowData.errorCount ? "component-error-icon" :
					rowData.warningCount ? "component-warning-icon" : "component-icon";
		}
		if(cellType == Constants.COMPILE_PACKAGE_NAME_COLUMN){
			if (rowData.packageType == "plugin") {
				return rowData.errorCount ? "plugin-error-icon" :
						rowData.warningCount ? "plugin-warning-icon" : "plugin-icon";
			}
			return rowData.errorCount ? "package-error-icon" :
					rowData.warningCount ? "package-warning-icon" : "package-icon";
		}
		if(cellType == Constants.COMPILE_FILE_NAME_COLUMN){
			var result;
			// if it has a sourceName, it's a java file, else it's a simple contribution
			if (rowData.sourceName) {
				result = "file-icon";
			} else {
				result = "compile-icon";
			}
			if (rowData.errorCount) {
				result += "-error";
			} else if (rowData.warningCount) {
				result += "-warning";
			}
			return result;
		}
		return "";
	}
	
});
	
})();
