/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.viewlets.web.ui.internal.utils.MetagroupsPreferenceProvider");

dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");
dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.utils.MetagroupsDialog");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "WorkItemsMessages");

// TODO: MP: Cache label?

(function() {

var CustomPreferenceProvider = com.ibm.team.dashboard.web.ui.CustomPreferenceProvider;
var MetagroupsDialog = com.ibm.team.workitem.viewlets.web.ui.internal.utils.MetagroupsDialog;

var messages = dojo.i18n.getLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "WorkItemsMessages");

dojo.declare("com.ibm.team.workitem.viewlets.web.ui.internal.utils.MetagroupsPreferenceProvider",
	CustomPreferenceProvider, {

	/*
	 * Properties
	 */		
		
	/*
 	 * @param Object args
 	 * 
 	 * @return void
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "StatisticsMessages");
		
		this._metagroups = [];
	},

	/*
 	 * @param String value
 	 * 
 	 * @return String
 	 */
	getLabel: function(value){
		
		// TEST
		//value = "Jazz User~~TestJazzUser@@Jazz Admin~~TestJazzAdmin@@Jazz Guest~~TestJazzGuest";
		
		var text;
		
		if (!value) {
			this._metagroups = [];
			text = this.messages.msgNoMetagroupsDefined;
		} else {
			this._metagroups = this._deserializeMetagroups(value);
			text = this._getMetagroupsLabel(this._metagroups);
		}
		
		return this._buildLabel(text);
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return String
 	 * @overrides super
 	 */
	getLoadingLabel: function() {
		return this._buildLabel(this.messages.msgLoading);
	},	
	
	/*
	 * @param Function onOk(String value, String prefId)
	 * @param Object settings
	 * 				. Object preferences
	 *					. <Type-1> <Preference-ID-1>
	 *					. <Type-2> <Preference-ID-2>
	 *					. ...
	 *					. <Type-N> <Preference-ID-N>
	 *				. Number scope
	 *				. String scopeItemId
	 *				. boolean showBackground
	 *				. String title
	 *				. Number trim
	 *				. boolean userTitle
	 * @param Object viewlet
	 * 
	 * @return void
	 * @overrides super
	 */	
	createChooser: function(onOk, settings, viewlet) {
		
		var dialog = new MetagroupsDialog({
			metagroups: this._metagroups,
			onOK: dojo.hitch(this, function(metagroups) {
				var value = this._serializeMetagroups(metagroups);
				onOk(value, viewlet._static._PREFERENCE_ID_METAGROUPS);
			})
		});
		
		dialog.show();
	},

	/*
 	 * @param String text
 	 * 
 	 * @return String
 	 */
	_buildLabel: function (text) {
		return "<input type='text' readonly='readonly' value='" + text.replace("'","&#39;") + "'/>";
	},
	
	/*
 	 * @param Array metagroups
 	 * 
 	 * @return String
 	 */
	_getMetagroupsLabel: function(metagroups){
		
		var result;
		var labels = [];
		
		for (var index = 0; index < metagroups.length; index++) {
			var group = metagroups[index];
			if (!group) {
				// TODO: MP: Error message?
				continue;
			} else if (!group.label) {
				// TODO: MP: Error message?
				continue;
			}
			labels.push(group.label);
		}
		
		if (labels.length > 0) {
			result = labels.join(", ");
		} else {
			// TODO: MP: Error message?
			result = this.messages.msgNoMetagroupsDefined;
		}
		
		return result;
	},	
	
	/*
 	 * @param Array metagroups
 	 * 
 	 * @return String
 	 */
	_serializeMetagroups: function(metagroups) {
		
		// Format:
		// [PatternName 1]~~[Pattern Value 1]@@[PatternName 2]~~[Pattern Value 2]@@[PatternName N]~~[Pattern Value N]
		
		var result;
		var patternSegments = [];
		
		for (var index = 0; index < metagroups.length; index++) {
			var group = metagroups[index];
			if (!group) {
				// TODO: MP: Error message?
				continue;
			} else if (!group.label) {
				// TODO: MP: Error message?
				continue;
			} else if (!group.pattern) {
				// TODO: MP: Error message?
				continue;
			}
			
			var segment = group.label + "~~" + group.pattern;
			patternSegments.push(segment);
		}
		
		if (patternSegments.length > 0) {
			result = patternSegments.join("@@");
		} else {
			// TODO: MP: Error message?
			result = "";
		}
		
		return result;		
	},	

	/*
 	 * @param String value
 	 * 
 	 * @return Array
 	 */
	_deserializeMetagroups: function(value) {
		
		// Format:
		// [PatternName 1]~~[Pattern Value 1]@@[PatternName 2]~~[Pattern Value 2]@@[PatternName N]~~[Pattern Value N]
		
		if (!value) {
			return [];
		}
		
		var patternSegments = value.split("@@");
		var metagroups = [];
		
		for (var index = 0; index < patternSegments.length; index++) {
			var pattern = patternSegments[index];
			if (!pattern) {
				// TODO: MP: Error message?
				continue;
			}
			
			var patternPair = pattern.split("~~", 2);
			if (patternPair.length !== 2) {
				// TODO: MP: Error message?
				continue;
			}
			
			var patternName = patternPair[0];
			var patternValue = patternPair[1];
			
			metagroups.push({
				label: patternName,
				pattern: patternValue
			});
		}
		
		return metagroups;
	}

});

})();
