/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.utils.WorkItemPicker");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.Dialog");

dojo.require("jazz.app.auth");
dojo.require("jazz.app.events");
dojo.require("jazz.app.proxy");
dojo.require("jazz.client.xhr");

dojo.require("jazz.ui.RemoteChooserContainer");

(function() {

dojo.declare("com.ibm.team.build.web.ui.internal.utils.WorkItemPicker", [dijit._Widget], {
	/*
     * Parameters for RemoteItemPicker:
     *   parentElement (req): the parent element to place the dialog as a child
     *   dialogTitle (req): dialog title
     *   secondaryTitle (req): secondary dialog title
     *   pickerURL (req): the picker url
     *   width (opt): the width of the dialog
     *   height (opt): the height of the dialog
     *   namespace: the namespace used for the result ('oslc_cm'...)
     *   onOk: (req) function(result, action) called when items have been selected. Result type is [ { "url", "label" } ]
	 */
	constructor: function(args) {
        if (!args.dialogTitle) throw new Error("Missing 'dialogTitle'");
        if (!args.namespace) throw new Error("Missing 'namespace'");
        if (!args.pickerURL) throw new Error("Missing 'pickerURL'");
        if (!dojo.isFunction(args.onOk)) throw new Error("Missing 'onOk'");
	},
	
	postCreate: function() {
	    var url= new dojo._Url(this.pickerURL);
	    var server= url.authority;

		this._initContentWidget();
	},
	_onData: function(data) {
	    if (data.length != 0) {
	      var response= dojo.fromJson(data);
	      var newItems= response[this.namespace + ":results"];
	      if (newItems instanceof Array && newItems.length > 0) {
	        var result= [];
	        for (var i= 0; i < newItems.length; i++) {
	          var item= newItems[i];
	          result.push({
	            url: item["rdf:resource"],
	            label: item[this.namespace + ":label"]
	          });
	        }
	        this.onOk(result);
	      }
	    }
		this.floatingPane.close();
	},
	_initContentWidget: function() {			
        var frameSrc= this.pickerURL;

		var frameContainerArgs = { 
            remoteLocation: frameSrc,
            onData: dojo.hitch(this, this._onData)
		};
				
		this.content = new jazz.ui.RemoteChooserContainer(frameContainerArgs);
		
		var floatingPaneParams = {
            primaryTitle: this.dialogTitle,
            secondaryTitle: this.secondaryTitle,
			width: this.width,
			height: this.height,
			onClose: dojo.hitch(this, this._handleClose),
			contentNode: this.content.domNode
		};		
	
		this.floatingPane = new jazz.ui.Dialog(floatingPaneParams);
	},
	
	_handleClose: function() {
		if(this.content)
			this.content.destroy();
	}
	
});

})();