/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
/**
 * @author bjiang
 */
dojo.provide("com.ibm.team.scm.web.ui.internal.view.browse.FilesFoldersView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.fx");
dojo.require("dojo.io.iframe");
dojo.require("dijit.form.CheckBox");
dojo.require("jazz.util.ResizeNotifier"); //$NON-NLS-1$
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.ui.internal.SimpleToolbar");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("dojo.date.locale");
dojo.require("com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.ResourceEditor");
dojo.require("com.ibm.team.scm.web.ui.internal.view.browse.FileFolderContentView");
dojo.require("com.ibm.team.scm.web.ui.internal.view.browse.HistoryView");
dojo.require("com.ibm.team.scm.web.ui.internal.widgets.OverlayIcon");

dojo.requireLocalization("com.ibm.team.scm.web.internal", "Messages");

(function(){ // module container
    var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var ScmUtil = com.ibm.team.scm.web.client.util;
    var ScmClient = com.ibm.team.scm.web.client.internal.ScmClient;
    var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var BUTTON_CONTEXT = "com.ibm.team.scm.web";
	var ADD_FILE_IMAGE =  "graphics/icons/addfile_co.gif";  // $NON-NLS-1$
	var NEW_TEXT_FILE_IMAGE =  "graphics/icons/createfile_co.gif";  // $NON-NLS-1$
	var EDIT_TEXT_FILE_IMAGE =  "graphics/icons/edit_co.gif";  // $NON-NLS-1$
	var ADD_FOLDER_IMAGE = "graphics/icons/addfolder_co.gif"; // $NON-NLS-1$
	var DOWNLOAD_IMAGE = "graphics/icons/download_co.gif"; // $NON-NLS-1$
	var UPLOAD_IMAGE =  "graphics/icons/upload_co.gif";  // $NON-NLS-1$
	var LOCK_IMAGE = "graphics/icons/locks.gif"; // $NON-NLS-1$
	var UNLOCK_IMAGE = "graphics/icons/unlock.gif"; // $NON-NLS-1$
	var DELETE_IMAGE = "graphics/icons/delete_edit.gif"; // $NON-NLS-1$
	var LOCK_DOWNLOAD_IMAGE = "graphics/icons/lockdownload_co.gif"; // $NON-NLS-1$
	var WARNING_IMAGE = "graphics/icons/warning_obj.gif"; // $NON-NLS-1$    
    
    /* defineWidget */
    dojo.declare("com.ibm.team.scm.web.ui.internal.view.browse.FilesFoldersView", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.team.scm.web", "ui/internal/view/browse/templates/FilesFoldersView.html"),
        
        TAB_NAME_OVERVIEW: "overview",
        TAB_NAME_HISTORY: "history",
        TAB_NAME_PROPERTIES: "properties",
        
        // object initializer
        constructor: function(args){
            this.streamWorkspaceView = args.parentView;
            this.pageController = this.streamWorkspaceView.controller;
            this.messages = dojo.i18n.getLocalization("com.ibm.team.scm.web.internal", "Messages");
			
			this._toolbarButtons = {};
            this._tabViewParts = {};
        },
        
        postCreate: function(){			
			this._toolbar = new jazz.ui.internal.SimpleToolbar();
			this._simpleToolbarContainer.appendChild(this._toolbar.domNode);
			
			// Toolbar buttons: "addFileButton", "newTextFileButton", "addFolderButton", 
			// "lockAnddownloadButton", "downloadButton", "uploadButton", 
			// "editTextFileButton", "lockButton", "unlockButton", "deleteButton"			
			
			this._toolbarButtons.addFileButton =  this._toolbar.addButton(this, this._handleFileAddBrowserClick,
				 {imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, ADD_FILE_IMAGE), 
				 	label: this.messages.StreamWorkspaceView_62});
			this._toolbarButtons.newTextFileButton =  this._toolbar.addButton(this, this._handleNewTextFileBrowserClick,
				 {imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, NEW_TEXT_FILE_IMAGE), 
				 	label: this.messages.StreamWorkspaceView_69});
			this._toolbarButtons.addFolderButton =  this._toolbar.addButton(this, this._handleFolderAddBrowserClick,
				 {imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, ADD_FOLDER_IMAGE), 
				 	label: this.messages.StreamWorkspaceView_63});
			this._toolbarButtons.lockAnddownloadButton =  this._toolbar.addButton(this, this._handleLockAndDownloadClick,
				 {imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, LOCK_DOWNLOAD_IMAGE), 
				 	label: this.messages.StreamWorkspaceView_68});
			this._toolbarButtons.downloadButton =  this._toolbar.addButton(this, this._handleDownloadClick,
				 {imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, DOWNLOAD_IMAGE), 
				 	label: this.messages.StreamWorkspaceView_55});
			this._toolbarButtons.uploadButton =  this._toolbar.addButton(this, this._handleUploadBrowserClick,
				 {imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, UPLOAD_IMAGE), 
				 	label: this.messages.StreamWorkspaceView_60});
			this._toolbarButtons.editTextFileButton =  this._toolbar.addButton(this, this._handleEditTextFileBrowserClick,
				 {imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, EDIT_TEXT_FILE_IMAGE), 
				 	label: this.messages.StreamWorkspaceView_70});
			this._toolbarButtons.lockButton =  this._toolbar.addButton(this, this._handleFileLockClick,
				 {imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, LOCK_IMAGE), 
				 	label: this.messages.StreamWorkspaceView_64});					
			this._toolbarButtons.unlockButton =  this._toolbar.addButton(this, this._handleFileUnlockClick,
				 {imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, UNLOCK_IMAGE), 
				 	label: this.messages.StreamWorkspaceView_65});					
			this._toolbarButtons.deleteButton =  this._toolbar.addButton(this, this._handleFileDeleteBrowserClick,
				 {imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, DELETE_IMAGE), 
				 	label: this.messages.StreamWorkspaceView_61});
        },
        
        destroy: function(){
        },
        
        setContent: function(args, subElements){
        
			this.versionableDTO = subElements;
			this.contentArgs = args;
			this._isFolder = this.versionableDTO.versionable["com.ibm.team.repository.typeName"] == "com.ibm.team.scm.Folder" ? true : false;
			
			this.refresh();
		},
		
		refresh: function(){       
			if(!this.versionableDTO)
				return;
				     
            //Clear current page view.
            dojox.data.dom.removeChildren(this.editBrowser);
            this._setTitleAndToolbar(this.versionableDTO, this.contentArgs.isComponent);
            this._setTabs(this.versionableDTO);
			if(!this.contentArgs.tab)
            	this.showTab(this.TAB_NAME_OVERVIEW, this.versionableDTO, this.contentArgs);
			else
            	this.showTab(this.contentArgs.tab, this.versionableDTO, this.contentArgs);
        },
        
        _setTitleAndToolbar: function(versionableDTO, isComponent){
			
			// disable all toolbar buttons, then enable appropriate ones.
			this._setToolbarButtonVisible(false);			
			dojox.data.dom.removeChildren(this.itemIcon);
			dojox.data.dom.removeChildren(this.itemNameNode);
			dojox.data.dom.removeChildren(this.validationMessage);
			dojo.removeClass(this.validationMessageSpan, "validationMessageWarning");
			dojo.addClass(this.validationMessageSpan, "validationMessageEmpty");
					
			var fromStream = versionableDTO.workspace.stream;
			var lockedBy = versionableDTO.versionablePlusDTO.lockedBy;
			var isLocked = lockedBy ? true : false;
							
			if (isComponent) {
				this.itemNameNode.appendChild(document.createTextNode(versionableDTO.component.name));
				this.itemIcon.appendChild(ScmUtil.getIconWithLockingState(ScmUtil.getIconSrc("component"), isLocked, lockedBy));
				if(fromStream)
					this._setToolbarButtonVisible(true, ["addFileButton", 
														 "newTextFileButton", 
														 "addFolderButton"]);
			}
			else if (versionableDTO.versionable["com.ibm.team.repository.typeName"] == "com.ibm.team.scm.Folder") {
				this.itemNameNode.appendChild(document.createTextNode(versionableDTO.versionable.name));
				this.itemIcon.appendChild(ScmUtil.getIconWithLockingState(ScmUtil.getIconSrc("folder"), isLocked, lockedBy));
				if(fromStream)
					this._setToolbarButtonVisible(true, ["addFileButton", 
														 "newTextFileButton", 
														 "addFolderButton", 
														 "deleteButton"]);
			}
			else if (versionableDTO.versionable["com.ibm.team.repository.typeName"] == "com.ibm.team.filesystem.FileItem") {
				this.itemNameNode.appendChild(document.createTextNode(versionableDTO.versionable.name));
				this.itemIcon.appendChild(ScmUtil.getDecoratedResourceIcon({ fileName : versionableDTO.versionable.name, isLocked : isLocked, lockedBy : lockedBy }));
				this._toolbarButtons.downloadButton.domNode.href = ScmUtil.getFileDownloadURI(versionableDTO.workspace.itemId, 
																							  versionableDTO.component.itemId, 
																							  versionableDTO.versionable.name, 
																							  versionableDTO.versionable.itemId, 
																							  null);
				
				if(fromStream)																						  
					this._setToolbarButtonVisible(true, ["lockAnddownloadButton", 
														 "downloadButton",
														 "uploadButton",
														 "editTextFileButton",
														 "lockButton",
														 "unlockButton",
														 "deleteButton"]);
				else // from workspace view
					this._setToolbarButtonVisible(true, ["downloadButton"]);
					
				
				//warning message if locked by other
				if (isLocked === true && lockedBy.itemId != ScmUtil.getLoggedInContributor().itemId) {
					this.validationMessageIcon.src = dojo.moduleUrl(BUTTON_CONTEXT, WARNING_IMAGE);
					
					if (lockedBy.emailAddress) {
						this.validationMessage.innerHTML = this.messages.FilesFoldersView_FileLocked_Warning_NoName;
						var userLink = document.createElement("a");
						dojo.addClass(userLink, "validationMessageWarningLink");
						userLink.href = "mailto:" + lockedBy.emailAddress + "?subject=" + dojo.string.substitute(this.messages.FilesFoldersView_FileLocked_Warning_MailSubject, [versionableDTO.component.name]);
						userLink.appendChild(document.createTextNode(lockedBy.name));
						if (lockedBy.name && lockedBy.userId) 
							userLink.title = dojo.string.substitute(this.messages.FilesFoldersView_FileLocked_Warning_Tooltip, [lockedBy.name, lockedBy.userId]);
						this.validationMessage.appendChild(userLink);
					}
					else {						
						this.validationMessage.appendChild(document.createTextNode(dojo.string.substitute(this.messages.FilesFoldersView_FileLocked_Warning, [lockedBy.name])));
					}
					
					dojo.removeClass(this.validationMessageSpan, "validationMessageEmpty");
					dojo.addClass(this.validationMessageSpan, "validationMessageWarning");
				}																	 
			}
        },
		
		/**
		 * Toogle the visibility of toolbar buttons.
		 * @param {Object} visible  
		 * @param {Object} buttonNames null or missing this param will update all buttons in the toolbar.
		 */
		_setToolbarButtonVisible: function(visible, buttonNames){
			var self = this;
			if(buttonNames) {
				dojo.forEach(buttonNames, function(button){
					if (visible) {
						self._toolbar.showItem(self._toolbarButtons[button]);
					} else {
						self._toolbar.hideItem(self._toolbarButtons[button]);
					}
				});
			}
			else {
				for(var i in this._toolbarButtons){
					if (visible) {
						self._toolbar.showItem(self._toolbarButtons[i]);
					} else {
						self._toolbar.hideItem(self._toolbarButtons[i]);
					}
				}
			}
		},
        
        _setTabs: function(versionableDTO){
			dojox.data.dom.removeChildren(this._tabLinkContainer);
			dojox.data.dom.removeChildren(this._currentTabContainer);
			
            // component and folder view has overview and history tabs
            this._createTabPart(this.TAB_NAME_OVERVIEW, this.messages.FilesFoldersView_DefaultTabTitleOverview);
            this._createTabPart(this.TAB_NAME_HISTORY, this.messages.FilesFoldersView_TabTitleHistory);
			
//            var filePropTab = this._createTabPart(this.TAB_NAME_PROPERTIES, this.messages.FilesFoldersView_TabTitleProperties);            
            // file view also has properties tab.
//            if (versionableDTO.versionable["com.ibm.team.repository.typeName"] == "com.ibm.team.filesystem.FileItem") 
//                filePropTab.style.visibility = '';
//            else 
//                filePropTab.style.visibility = 'hidden';
            
            // update the tab URLs
            var tabLinks = this._tabLinkContainer.getElementsByTagName("a");
            var self = this;
            dojo.forEach(tabLinks, function(link){
                var i = link.href.indexOf("#");
                if (i != -1) 
                    link.href = link.href.substring(0, i);
				
                if (link.__jazzTabName == self.TAB_NAME_OVERVIEW) 
                	link.href += ScmUtil.getBrowseElementURI(versionableDTO.workspace.itemId, versionableDTO.component.itemId, versionableDTO.path);
                else 
                    link.href += ScmUtil.getBrowseElementTabURI(versionableDTO.workspace.itemId, versionableDTO.component.itemId, versionableDTO.path, link.__jazzTabName);
            });
        },
        
        /**
         * Create a subtab for the files folders view.
         *
         * @param {String} logicalName this name to identify the editor part via
         * 	this._tabViewParts and also the name to specify which widget to show
         *  in the viewWorkItem URL.
         * @param {String} tabLinkLabel the label of the hyperlink that allows users
         *  to view the particular tab.
         * @param {widget} widget is a widget to associate
         *  with the reference from this._tabViewParts[logicalName].
         * @param {Object} createArgs arguments with which to initialize the widget
         */
        _createTabPart: function(logicalName, tabLinkLabel){
            if (logicalName) 
                logicalName = escape(logicalName).replace(/\%/g, "_"); // NLS tab names
            //			this._tabViewParts[logicalName]= widget;
            // create hyperlink for each tab
            
            try {
                if (this._tabLinkContainer) {
                    for (var i = 0; i < this._tabLinkContainer.childNodes.length; i++) {
                        if (this._tabLinkContainer.childNodes[i] && this._tabLinkContainer.childNodes[i].__jazzTabName == logicalName) {
                            return this._tabLinkContainer.childNodes[i]; // already exists
                        }
                    }
                }
            } 
            catch (e) {
                console.debug(dojo.string.substitute(this.messages.errorCreatingTab, [logicalName]))
            };
            
            var link = document.createElement("a");
            dojo.addClass(link, "tab");
            link.href = "#";
            link.__jazzTabName = logicalName;
            link.title = tabLinkLabel;
            
            
            var innerWrapper = document.createElement("div");
            innerWrapper.className = "innerWrapper";
            
            var topWrapper = document.createElement("div");
            topWrapper.className = "topWrapper";
            
            var topBorderDiv = document.createElement("div");
            topBorderDiv.className = "topBorder";
            topWrapper.appendChild(topBorderDiv);
            
            var cornerDiv = document.createElement("div");
            cornerDiv.className = "corners";
            topWrapper.appendChild(cornerDiv);
            
            link.appendChild(topWrapper);
            
            var titleDiv = document.createElement("div");
            titleDiv.className = "title";
            titleDiv.innerHTML = tabLinkLabel;
            innerWrapper.appendChild(titleDiv);
            
            link.appendChild(innerWrapper);
            
            this._tabLinkContainer.appendChild(link);
            return link;
        },
        /**
         * Displays the tab specified by <code>tabName</code>.
         *
         * @param {String} tabName the logical name of the tab to display.
         */
        showTab: function(tabName, versionableDTO, args){
        
            // If the tab name is in the window url, override the method call.
            //			var urlTab = WorkItemUtil.getUrlParam(window.location.href, "tab");
            //			if(urlTab) {
            //				tabName = urlTab;
            //			} else {
            //				tabName = tabName || this.getDefaultTabName();
            //			}
            
            //			if(!this._tabViewParts[tabName]) {
            //				if(this.knownTabs[tabName] ){
            //					tabName = this.knownTabs[tabName];
            //				} else {
            //	 				tabName = this.getDefaultTabName();
            //				}
            //			}
            
            //			if(!this._tabViewParts[tabName]) return; // click during refresh
			
			tabName = tabName.toLowerCase();
            if (this._tabViewParts[tabName] && this._tabViewParts[tabName].showWidget) { // use cached widget
                this._tabViewParts[tabName].showWidget(versionableDTO, args);
            }
            else { // create new widget
            	var contentWidget = null;
            	if (tabName == this.TAB_NAME_OVERVIEW) {
					contentWidget = this._getOverviewWidget();
					contentWidget.showWidget(versionableDTO, args);
				}
				else if (tabName == this.TAB_NAME_HISTORY) {
					contentWidget = this._getHistoryView();					
					contentWidget.showWidget(versionableDTO, args);
				}
				
                this._tabViewParts[tabName] = contentWidget;
            }
            
            // swap in contents
            if (this._currentTabContainer.firstChild) {
                this._currentTabContainer.replaceChild(this._tabViewParts[tabName].domNode, this._currentTabContainer.firstChild);
            }
            else {
                this._currentTabContainer.appendChild(this._tabViewParts[tabName].domNode);
            }
            
            // update tab selection
            var links = this._tabLinkContainer.childNodes;
            dojo.forEach(links, function(link){
                if (link.__jazzTabName === tabName) 
                    dojo.addClass(link, "Selected");
                else {
                    dojo.removeClass(link, "Selected");
                }
            });
            
            //			if(tabName != this._currentTab) {
            //				this._sendChangedEvent("Tab Changed");
            //			}
            
            // store a reference to the current tab
            this._currentTab = tabName;
        },
		
		_getOverviewWidget: function(){
			if(!this._overviewWidget)
				this._overviewWidget = new com.ibm.team.scm.web.ui.internal.view.browse.FileFolderContentView({
						parentView: this.streamWorkspaceView
					});
			return this._overviewWidget;
		},
		
		_getHistoryView: function(){
			if(!this._historyView)
				this._historyView = new com.ibm.team.scm.web.ui.internal.view.browse.HistoryView({controller: this.pageController});
				
			return this._historyView;
		},
        
		// TO BE REMOVED
        _addFolderEditLinks: function(subElements){
            var workspaceId = subElements.workspace.itemId;
            var componentId = subElements.component.itemId;
            
            var table = document.createElement("table");
            table.className = "show-options";
            var tbody = document.createElement("tbody");
            var tr = document.createElement("tr");
            this.streamWorkspaceView._addHistoryLinkImpl(subElements, tr);
            /**
             * Add a "Create Folder", "Create File" and "Delete" link for a folder under a stream.
             */
            var td = document.createElement("td");
            var img = document.createElement("img");
            img.src = ScmUtil.getIconSrc("crtWorkItem");
            img.className = "headerIcon";
            
            var anchor = document.createElement("a");
            var text = document.createTextNode(this.messages.StreamWorkspaceView_62);
            anchor.appendChild(img);
            anchor.appendChild(text);
            anchor.href = "#add_file"; //enable the pointer
            var self = this;
            this.connect(anchor, "onclick", function(event){
                dojo.stopEvent(event);
                self._handleFileAddBrowserClick(subElements);
            });
            
            td.appendChild(anchor);
            tr.appendChild(td);
            
            var td = document.createElement("td");
            var img = document.createElement("img");
            img.src = ScmUtil.getIconSrc("include_obj");
            img.className = "headerIcon";
            
            var anchor = document.createElement("a");
            var text = document.createTextNode(this.messages.StreamWorkspaceView_63);
            anchor.appendChild(img);
            anchor.appendChild(text);
            anchor.href = "#add_folder"; //enable the pointer
            var self = this;
            this.connect(anchor, "onclick", function(event){
                dojo.stopEvent(event);
                self._handleFolderAddBrowserClick(subElements);
            });
            
            td.appendChild(anchor);
            tr.appendChild(td);
            
            // Add delete link on non root folder
            if (subElements.versionable.parentId) {
                var td = document.createElement("td");
                var img = document.createElement("img");
                img.src = ScmUtil.getIconSrc("delete_edit");
                img.className = "headerIcon";
                
                var anchor = document.createElement("a");
                var text = document.createTextNode(this.messages.StreamWorkspaceView_61);
                anchor.appendChild(img);
                anchor.appendChild(text);
                anchor.href = "#delete"; // enable the pointer
                var self = this;
                this.connect(anchor, "onclick", function(event){
                    dojo.stopEvent(event);
                    self._handleFolderDeleteBrowserClick(subElements);
                });
                
                td.appendChild(anchor);
                tr.appendChild(td);
            }
            tbody.appendChild(tr);
            table.appendChild(tbody);
            this.streamWorkspaceView.changeOverview.appendChild(table);
        },
        
		// TO BE REMOVED
        _addFileEditLinks: function(versionableDTO){
            var workspaceId = versionableDTO.workspace.itemId;
            var componentId = versionableDTO.component.itemId;
            var table = document.createElement("table");
            table.className = "show-options";
            var tbody = document.createElement("tbody");
            var tr = document.createElement("tr");
            var td = document.createElement("td");
            td.innerHTML = this.messages.StreamWorkspaceView_52;
            td.className = "history-header";
            tr.appendChild(td);
            var td = document.createElement("td");
            var img = document.createElement("img");
            img.src = ScmUtil.getIconSrc("history");
            img.className = "headerIcon";
            if (versionableDTO.path == "/") {
                //Add a link to the history of the component
                var anchor = document.createElement("a");
                if (workspaceId != null) {
                    anchor.href = (ScmUtil.getComponentHistoryURI(workspaceId, componentId));
                }
                anchor.title = dojo.string.substitute(this.messages.StreamWorkspaceView_53, [versionableDTO.component.name]);
                var text = document.createTextNode(this.messages.StreamWorkspaceView_54);
                anchor.appendChild(img);
                anchor.appendChild(text);
            }
            else {
                //Add a link to the history of the item
                var anchor = document.createElement("a");
                if (workspaceId != null) {
                    anchor.href = (ScmUtil.getItemHistoryURI(workspaceId, componentId, versionableDTO.versionable["com.ibm.team.repository.typeName"] + "/" + versionableDTO.versionable.itemId));
                }
                anchor.title = dojo.string.substitute(this.messages.StreamWorkspaceView_53, [versionableDTO.versionable.name]);
                var text = document.createTextNode(this.messages.StreamWorkspaceView_54);
                anchor.appendChild(img);
                anchor.appendChild(text);
            }
            td.appendChild(anchor);
            tr.appendChild(td);

            var td = document.createElement("td");
            var img = document.createElement("img");
            img.src = ScmUtil.getIconSrc("lockdownload_co");
            img.className = "headerIcon";
            
            var anchor = document.createElement("a");
            var text = document.createTextNode(this.messages.StreamWorkspaceView_68);
            anchor.appendChild(img);
            anchor.appendChild(text);
            anchor.href = "#lockAndDownload";
            var self = this;
            this.connect(anchor, "onclick", function(event){
                dojo.stopEvent(event);
				self._handleLockAndDownloadClick(versionableDTO);
            });
            
            td.appendChild(anchor);
            tr.appendChild(td);
            
            var td = document.createElement("td");
            var img = document.createElement("img");
            img.src = ScmUtil.getIconSrc("download_co");
            img.className = "headerIcon";
            
            var anchor = document.createElement("a");
            anchor.href = ScmUtil.getFileDownloadURI(workspaceId, componentId, versionableDTO.versionable.name, versionableDTO.versionable.itemId, null);
            var text = document.createTextNode(this.messages.StreamWorkspaceView_55);
            anchor.appendChild(img);
            anchor.appendChild(text);
            
            td.appendChild(anchor);
            tr.appendChild(td);
            
            /**
             * Add a "Upload" and "Delete" link for a file under a stream.
             */
            var td = document.createElement("td");
            var img = document.createElement("img");
            img.src = ScmUtil.getIconSrc("upload_co");
            img.className = "headerIcon";
            
            var anchor = document.createElement("a");
            var text = document.createTextNode(this.messages.StreamWorkspaceView_60);
            anchor.appendChild(img);
            anchor.appendChild(text);
            anchor.href = "#upload"; //enable the pointer
            var self = this;
            this.connect(anchor, "onclick", function(event){
                dojo.stopEvent(event);
                self._handleUploadBrowserClick(versionableDTO);
            });
            
            td.appendChild(anchor);
            tr.appendChild(td);
            
            var td = document.createElement("td");
            var img = document.createElement("img");
            img.src = ScmUtil.getIconSrc("delete_edit");
            img.className = "headerIcon";
            
            var anchor = document.createElement("a");
            var text = document.createTextNode(this.messages.StreamWorkspaceView_61);
            anchor.appendChild(img);
            anchor.appendChild(text);
            anchor.href = "#delete"; // enable the pointer
            var self = this;
            this.connect(anchor, "onclick", function(event){
                dojo.stopEvent(event);
                self._handleFileDeleteBrowserClick(versionableDTO);
            });
            
            td.appendChild(anchor);
            tr.appendChild(td);
            
            var td = document.createElement("td");
            var img = document.createElement("img");
            img.src = ScmUtil.getIconSrc("locks");
            img.className = "headerIcon";
            
            var anchor = document.createElement("a");
            var text = document.createTextNode(this.messages.StreamWorkspaceView_64);
            anchor.appendChild(img);
            anchor.appendChild(text);
            anchor.href = "#lock"; // enable the pointer
            var self = this;
            this.connect(anchor, "onclick", function(event){
                dojo.stopEvent(event);
                self._handleFileLockClick(versionableDTO);
            });
            
            td.appendChild(anchor);
            tr.appendChild(td);
            
            var td = document.createElement("td");
            var img = document.createElement("img");
            img.src = ScmUtil.getIconSrc("unlock");
            img.className = "headerIcon";
            
            var anchor = document.createElement("a");
            var text = document.createTextNode(this.messages.StreamWorkspaceView_65);
            anchor.appendChild(img);
            anchor.appendChild(text);
            anchor.href = "#unlock"; // enable the pointer
            var self = this;
            this.connect(anchor, "onclick", function(event){
                dojo.stopEvent(event);
                self._handleFileUnlockClick(versionableDTO);
            });
            
            td.appendChild(anchor);
            tr.appendChild(td);
            
            tbody.appendChild(tr);
            table.appendChild(tbody);
            this._header.appendChild(table);
        },
        
        /*
         * Editors
         */
        _handleUploadBrowserClick: function(){
            // Allow only one upload resources browser widget
            if (this.editBrowser.hasChildNodes()) 
                return;
            var child = document.createElement("div");
            this.editBrowser.appendChild(child);
			var self = this;
            var editor = new com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.ResourceEditor({
                mode: "SINGLE_FILE_UPLOAD",
                versionableDTO: this.versionableDTO,
				pageController: this.pageController,
				onResourceEditorSaved: dojo.hitch(this, this.reload)
            }, child);
            editor.startup();
        },
        _handleEditTextFileBrowserClick: function(){
            // Allow only one upload resources browser widget
            if (this.editBrowser.hasChildNodes()) 
                return;
            var child = document.createElement("div");
            this.editBrowser.appendChild(child);
			var self = this;
            var editor = new com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.ResourceEditor({
                mode: "SINGLE_FILE_UPLOAD",
				hint: "TEXT_EDIT",
                versionableDTO: this.versionableDTO,
				pageController: this.pageController,
				onResourceEditorSaved: dojo.hitch(this, this.reload)
            }, child);
            editor.startup();
        },		
        _handleFileAddBrowserClick: function(){
            // Allow only one upload resources browser widget
            if (this.editBrowser.hasChildNodes()) 
                return;
            var child = document.createElement("div");
            this.editBrowser.appendChild(child);
            var editor = new com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.ResourceEditor({
                mode: "SINGLE_FILE_ADD",
                versionableDTO: this.versionableDTO,
				pageController: this.pageController,
				onResourceEditorSaved: dojo.hitch(this, this.refreshTabs)
            }, child);
//			dojo.connect(editor, "_onResourceEditorSaved", this, "refreshTabs");
            editor.startup();
        },
        _handleNewTextFileBrowserClick: function(){
            // Allow only one upload resources browser widget
            if (this.editBrowser.hasChildNodes()) 
                return;
            var child = document.createElement("div");
            this.editBrowser.appendChild(child);
            var editor = new com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.ResourceEditor({
                mode: "SINGLE_FILE_ADD",
				hint: "TEXT_EDIT",
                versionableDTO: this.versionableDTO,
				pageController: this.pageController,
				onResourceEditorSaved: dojo.hitch(this, this.refreshTabs)
            }, child);
            editor.startup();
        },
        _handleFileDeleteBrowserClick: function(){
            // Allow only one edit resources browser widget
            if (this.editBrowser.hasChildNodes()) 
                return;
            var child = document.createElement("div");
            this.editBrowser.appendChild(child);
            var editor = new com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.ResourceEditor({
                mode: this._isFolder ? "SINGLE_FOLDER_DELETE" : "SINGLE_FILE_DELETE",
                versionableDTO: this.versionableDTO,
				pageController: this.pageController,
				onResourceEditorSaved: dojo.hitch(this, this.reload, ScmUtil.parseParentPath(this.versionableDTO.path))
            }, child);
            editor.startup();
        },
        _handleFolderAddBrowserClick: function(){
            // Allow only one edit resources browser widget
            if (this.editBrowser.hasChildNodes()) 
                return;
            var child = document.createElement("div");
            this.editBrowser.appendChild(child);
            var editor = new com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.ResourceEditor({
                mode: "SINGLE_FOLDER_ADD",
                versionableDTO: this.versionableDTO,
				pageController: this.pageController,
				onResourceEditorSaved: dojo.hitch(this, this.refreshTabs)
            }, child);
            editor.startup();
        },
        _handleFolderDeleteBrowserClick: function(){
            // Allow only one edit resources browser widget
            if (this.editBrowser.hasChildNodes()) 
                return;
            var child = document.createElement("div");
            this.editBrowser.appendChild(child);
            var editor = new com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.ResourceEditor({
                mode: "SINGLE_FOLDER_DELETE",
                versionableDTO: this.versionableDTO,
				pageController: this.pageController,
				onResourceEditorSaved: dojo.hitch(this, this.reload, ScmUtil.parseParentPath(this.versionableDTO.path))
            }, child);
            editor.startup();
        },
        _handleFileLockClick: function(){
			var d;
            PlatformUI.getWorkbench().showMessage(this.messages.StreamWorkspaceView_66);
            var self = this;
			var args = {
                operation: ["acquire"],
                streamId: this.versionableDTO.workspace.itemId,
                componentId: this.versionableDTO.component.itemId,
                versionableNamespace: ["com.ibm.team.filesystem"],
                versionableItemType: ["FileItem"],
                versionableItemId: [this.versionableDTO.versionable.itemId]
            };
			d = new dojo.Deferred();
            ScmClient.postUpdateLocks(new ServiceResponseHandler({
                handler: function(){
                    PlatformUI.getWorkbench().hideMessageArea();
                    // toggle lock and unlock action states					
					self.pageController.onUpdateLocks(args);
					self._refreshItemIconLockingStatus(true, ScmUtil.getLoggedInContributor());
					d.callback(true);
                },
                error: function(error){
                    if (error.message !== '') {
                        alert(error.message);
                    }
                    PlatformUI.getWorkbench().hideMessageArea();
					d.errback(new Error(""));
                }
            }, 'handler', 'error'), args);
			return d;
        },
        _handleFileUnlockClick: function(){
            PlatformUI.getWorkbench().showMessage(this.messages.StreamWorkspaceView_67);
            var self = this;
			var args = {
                operation: ["release"],
                streamId: [this.versionableDTO.workspace.itemId],
                componentId: [this.versionableDTO.component.itemId],
                versionableNamespace: ["com.ibm.team.filesystem"],
                versionableItemType: ["FileItem"],
                versionableItemId: [this.versionableDTO.versionable.itemId]
            };
            ScmClient.postUpdateLocks(new ServiceResponseHandler({
                handler: function(){
                    PlatformUI.getWorkbench().hideMessageArea();
                    // toggle lock and unlock action states
					self.pageController.onUpdateLocks(args);
					self._refreshItemIconLockingStatus(false);
                },
                error: function(error){
                    if (error.message !== '') {
                        alert(error.message);
                    }
                    PlatformUI.getWorkbench().hideMessageArea();
                }
            }, 'handler', 'error'), args);
        },
        _handleLockAndDownloadClick: function(){
			this._handleFileLockClick(this.versionableDTO)
			.addCallback(dojo.hitch(this, function () {
				var href = ScmUtil.getFileDownloadURI(this.versionableDTO.workspace.itemId, this.versionableDTO.component.itemId, this.versionableDTO.versionable.name, this.versionableDTO.versionable.itemId, null);
				dojo.io.iframe.send({
					url : href,
					timeout : 2000 
				});
			}));
        },
		_refreshItemIconLockingStatus: function(isLocked, byContributor) {
			var icon = this.itemIcon.firstChild;
			if(icon) {	
				var src = null;
				// get the path of the base icon 
				if(icon.firstElementChild)	
					src = icon.firstElementChild.src; //from the OverlayIcon (when there's an overlay)
				else
					src = icon.src;		// or original icon (without overlay)
					
				if (src) {
					dojox.data.dom.removeChildren(this.itemIcon);
					this.itemIcon.appendChild(ScmUtil.getIconWithLockingState(src, isLocked, byContributor));
				}
			}
		},
		
		refreshTabs: function() {
			this._getOverviewWidget().reload();
			this._getHistoryView().refresh();
		},
		
		reload: function(path) {
			if(!this.versionableDTO)
				return;
				 
			var pathToLoad = path ? path : this.versionableDTO.path;
			
			// problem with this approach: 1. history tab flip to overview 2. performance esp. when there's a lot to reload in left hand navigator.
			var href = ScmUtil.getBrowseElementURI(this.versionableDTO.workspace.itemId, this.versionableDTO.component.itemId, pathToLoad);
			if (this._stringEndsWith(location.href, href)) {
				// force a reload of the current view
				PlatformUI.getWorkbench()._processUri(new dojo._Url(location.href));
			} else {
				// reload using the new file name 
				location.href = href;
			}
			
			// TODO: Breadcrumb doesn't get refreshed by this yet.
//			this._retrieveVersionable(this.versionableDTO.workspace.itemId, 
//								this.versionableDTO.component.itemId, 
//								pathToLoad, 
//								dojo.hitch(this, this.refresh), 
//								this);
		},
		
		/**
		 * Returns true if string1 ends with string2
		 */
		_stringEndsWith: function (string1, string2) {
			var index;
			index = string1.indexOf(string2);
			return index >= 0 && string2.length === string1.length - index;
		},
		
		_retrieveVersionable: function(workspaceId, componentId, path, successCallback, callbackParamObj) {
			checkNotNull(workspaceId, "workspaceId");
			checkNotNull(componentId, "componentId");
			checkNotNull(path, "path");
			
			var that = this;
			
			PlatformUI.getWorkbench().showMessage(this.messages.FetchingItem);
			var serviceObject = {
				success: function (versionablePlusDTO) {	
					PlatformUI.getWorkbench().hideMessageArea();
					
					if (versionablePlusDTO && versionablePlusDTO.versionableDTO) {
						var versionableDTO = versionablePlusDTO.versionableDTO;
						versionablePlusDTO.versionableDTO = null;
						versionableDTO.versionablePlusDTO = versionablePlusDTO;
						
						//cache this versionableDTO
						callbackParamObj.versionableDTO = versionableDTO;
						successCallback();
					}
				},
				error: function (error) { 
					PlatformUI.getWorkbench().hideMessageArea();
					if (that.pageController) {
						if (error.message !== "") {
							that.pageController._handleError(dojo.string.substitute(that.messages.ScmPageController_12, [ error.message, error.status ]));
						}
						else {
							that.pageController._handleError(that.messages.ScmPageController_13);
						}
					}
				}
			};							
						
			ScmClient.getVersionable(new ServiceResponseHandler(serviceObject, "success", "error"), 
									{workspace: ScmClient.WORKSPACE_PREFIX + workspaceId, component: ScmClient.COMPONENT_PREFIX + componentId}, 
									true, 
									path);
		}
        
    });
    
})();
