/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
/**
 * @author bjiang
 */

dojo.provide("com.ibm.team.scm.web.ui.internal.view.browse.HistoryView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dijit._base.scroll");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.team.scm.web.client.util");
dojo.require("com.ibm.team.scm.web.client.internal.ScmClient");
dojo.require("com.ibm.team.scm.web.ui.internal.view.browse.FileDiffView");
dojo.require("dojox.string.BidiComplex");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.requireLocalization("com.ibm.team.scm.web.internal", "Messages");

(function() { // module container

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ScmUtil					= com.ibm.team.scm.web.client.util;
var ScmClient				= com.ibm.team.scm.web.client.internal.ScmClient;
var	PlatformUI 				= net.jazz.ajax.ui.PlatformUI;
var TAB_NAME_HISTORY		= "history";
	
dojo.declare(
	"com.ibm.team.scm.web.ui.internal.view.browse.HistoryView",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.scm.web", "ui/internal/view/browse/templates/HistoryView.html"),		
		DEFAULT_PAGE_SIZE: 25,
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.scm.web.internal", "Messages");
			this.controller = args.controller;
			this.versionableDTO = null;
			this.args = null;
			this.currentConfigPageSize = this.DEFAULT_PAGE_SIZE;
		},
		
		postCreate: function() {
		},
		
		showWidget: function(versionableDTO, args) {
			this.versionableDTO = versionableDTO; //cache it and avoid re-render the UI next time.
			this.args = args;
			this.refresh();
		},
		
		refresh: function() {
			if(!this.versionableDTO)
				return;
				
			dojox.data.dom.removeChildren(this.tableHeading);
			dojox.data.dom.removeChildren(this.componentList);
			dojox.data.dom.removeChildren(this.pagingControl);
			
			var workspaceItemId = this.versionableDTO.workspace.itemId;
			var componentItemId = this.versionableDTO.component.itemId;
			var oid = this.versionableDTO.versionable["com.ibm.team.repository.typeName"] + "/" + this.versionableDTO.versionable.itemId;
			var isComponent = this.args.isComponent;
			
			var op = null;
			if(isComponent)
				op = this.controller.createOperation(this.messages.ScmPageController_31, "view component history");
			else
				op = this.controller.createOperation(this.messages.ScmPageController_32, "view item history");
			this._currentOperation= op;
			this.controller.startOperation(op);
	
			var self= this;
			var serviceObject= {
				viewHistoryHandler: function(historyPlusDTO) {
					checkNotNull(historyPlusDTO, "historyPlusDTO");
					
					if (historyPlusDTO) {
						PlatformUI.getWorkbench().setTitle(historyPlusDTO.historyDTO.workspace.name);
						if(isComponent)
							self._displayComponentHistory(historyPlusDTO);
						else
							self._displayItemHistory(historyPlusDTO, self.versionableDTO); //Display the item history
					}else {
						alert(dojo.string.substitute(self.messages.ScmPageController_14, [ oid ]));
					}
					
					if (op && op == self._currentOperation) {
						self.controller.finishOperation(op);
						self._currentOperation= null;
					}
				},
				error: function(error) { 
					if (error.message != "") {
						self._handleError(dojo.string.substitute(isComponent ? self.messages.ScmPageController_15 : self.messages.ScmPageController_17, [ error.message , error.status ]));
					}
					else {
						self._handleError(isComponent ? self.messages.ScmPageController_16 : self.messages.ScmPageController_18);
					}
					if (op && op == self._currentOperation) {
							self.controller.finishOperation(op);
							self._currentOperation= null;
					}
				}
			};
			
			if (!isComponent) {
				ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewHistoryHandler", "error"), {
					workspaceItemId: workspaceItemId,
					componentItemId: componentItemId,
					oid: oid
				});
			}
			else {
				this.currentConfigPageSize = this.args.n ? this.args.n : this.DEFAULT_PAGE_SIZE;
				//"Older" link in the history view is selected.
				if (this.args.first) {
					ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewHistoryHandler", "error"), {
						workspaceItemId: workspaceItemId,
						componentItemId: componentItemId,
						first: this.args.first,
						n: this.currentConfigPageSize
					});
				}
				//"Newer" link in the history view is selected.
				else if (this.args.last) {
					ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewHistoryHandler", "error"), {
						workspaceItemId: workspaceItemId,
						componentItemId: componentItemId,
						last: this.args.last,
						n: this.currentConfigPageSize
					});
				}
				//Default history link selection.
				else {
					ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewHistoryHandler", "error"), {
						workspaceItemId: workspaceItemId,
						componentItemId: componentItemId,
						n: this.currentConfigPageSize
					});
				}
			}
		},
		
		/**
		 * Sets the content of the main page view to either display components, files/folders, file content,
		 * component history, item history, active team changes, or change sets.
		 * 
		 * @param {Object} args
		 * @param {Object} subElements
		 */
		// Reserved from StreamWorkspaceView.js.  TO BE REFACTORED.
		_displayComponentHistory: function(historyPlusDTO) {			
			var workspace = historyPlusDTO.historyDTO.workspace;
			var component = historyPlusDTO.historyDTO.component;
			
			var tr = document.createElement("tr");
			var th = document.createElement("th");
			th.setAttribute("width", "100%");
			th.innerHTML = this.messages.StreamWorkspaceView_30;
			tr.appendChild(th);
			this.tableHeading.appendChild(tr);
			
			var changeSets = historyPlusDTO.historyDTO.changeSets;
			
			for(var i=0; i<changeSets.length; i++) {
				//Global row and data cell containing each change set entry
				var tr= document.createElement("tr");
				var td= document.createElement("td");
				td.setAttribute("width", "100%");
				var div= document.createElement("div");
				if(i%2==0) {
					div.className = "history-div-even";
				}
				else {
					div.className = "history-div-odd";
				}
				
				//Inner table containing the "Comment" field of a change set
				var table = document.createElement("table");
				table.setAttribute("width", "100%");
				table.className = "nested-table";
				var tbody = document.createElement("tbody");
				var tri = document.createElement("tr");
				var tdi = document.createElement("td");
				tdi.className = "history-inner-table";
				tdi.setAttribute("width", "5%");
				tdi.innerHTML = this.messages.StreamWorkspaceView_31;
				tri.appendChild(tdi);
								
				var tdi = document.createElement("td");
				var anchor = document.createElement("a");
				anchor.href = (ScmUtil.getChangeSetURI(workspace.itemId, changeSets[i].itemId, "1"));
				anchor.title = this.messages.StreamWorkspaceView_32;
				if (changeSets[i].comment && changeSets[i].comment != "" && changeSets[i].comment.length > 0) {
					var text = document.createTextNode(changeSets[i].comment);
				}
				else {			
					var text = document.createTextNode(this.messages.StreamWorkspaceView_33);
				}
				anchor.appendChild(text);
				tdi.appendChild(anchor);
				
				tri.appendChild(tdi);
				tbody.appendChild(tri);
				table.appendChild(tbody);
				div.appendChild(table);
				td.appendChild(div);
				
				//Inner table containing the "Reasons" (work-items) field of a change set
				var table = document.createElement("table");
				var tbody = document.createElement("tbody");
				table.className = "nested-table";
				table.setAttribute("width", "50%");	
				
				if(changeSets[i].reasons) {
					for(var j=0; j<changeSets[i].reasons.length; j++) {
						var tri = document.createElement("tr");
						var tdi = document.createElement("td");
						tdi.setAttribute("width", "7%");
						if(j==0) {
							tdi.className = "history-inner-table";
							tdi.innerHTML = this.messages.StreamWorkspaceView_34;
						}
						else {
							tdi.innerHTML = "";
						}
						tri.appendChild(tdi);
						var tdii = document.createElement("td");
						tdii.appendChild(ScmUtil.createReasonElement(changeSets[i].reasons[j]));
						tri.appendChild(tdii);
						tbody.appendChild(tri);
						table.appendChild(tbody);
					}
				}
				else {
					var tri = document.createElement("tr");
					var tdi = document.createElement("td");
					tdi.setAttribute("width", "7%");
					tdi.className = "history-inner-table";
					tdi.innerHTML = this.messages.StreamWorkspaceView_34;
					tri.appendChild(tdi);
					
					var tdi = document.createElement("td");
					tdi.innerHTML = this.messages.StreamWorkspaceView_35;
					tri.appendChild(tdi);
					
					tbody.appendChild(tri);
					table.appendChild(tbody);
				}
				
				div.appendChild(table);
				td.appendChild(div);
				
				//Inner table containing "Created by", "Added by", "Added on", "Modified on" fields of a change set
				var table = document.createElement("table");
				var tbody = document.createElement("tbody");
				table.className = "nested-table";
				table.setAttribute("width", "50%");
				var tri = document.createElement("tr");
				
				//"Created by" field
				var tdi = document.createElement("td");
				tdi.className = "history-inner-table";
				tdi.setAttribute("width", "7%");
				tdi.innerHTML = this.messages.StreamWorkspaceView_36;
				tri.appendChild(tdi);
				var tdi = document.createElement("td");
				tdi.setAttribute("width", "15%");
				tdi.appendChild(ScmUtil.createContributorElement(changeSets[i].author));
				tri.appendChild(tdi);
				tbody.appendChild(tri);
				
				//"Modified on" field
				var tdi = document.createElement("td");
				tdi.className = "history-inner-table";
				tdi.setAttribute("width", "7%");
				tdi.innerHTML = this.messages.StreamWorkspaceView_37;
				tri.appendChild(tdi);
				var tdi = document.createElement("td");
				var modifiedDateStr= ScmUtil._getDateString(changeSets[i].dateModified);
				var text = document.createTextNode(modifiedDateStr);
				tdi.appendChild(text);
				tri.appendChild(tdi);
				tbody.appendChild(tri);
				
				//"Added by" field
				var tri = document.createElement("tr");
				var tdi = document.createElement("td");
				tdi.className = "history-inner-table";
				tdi.setAttribute("width", "7%");
				tdi.innerHTML = this.messages.StreamWorkspaceView_38;
				tri.appendChild(tdi);
				var tdi = document.createElement("td");
				tdi.setAttribute("width", "15%");
				tdi.appendChild(ScmUtil.createContributorElement(changeSets[i].addedBy));
				tri.appendChild(tdi);
				
				//"Added on" field
				var tdi = document.createElement("td");
				tdi.className = "history-inner-table";
				tdi.setAttribute("width", "7%");
				tdi.innerHTML = this.messages.StreamWorkspaceView_39;
				tri.appendChild(tdi);
				var tdi = document.createElement("td");
				var addedDateStr= ScmUtil._getDateString(changeSets[i].dateAdded);
				var text = document.createTextNode(addedDateStr);
				tdi.appendChild(text);
				tri.appendChild(tdi);
				tbody.appendChild(tri);
				
				table.appendChild(tbody);
				div.appendChild(table);
				td.appendChild(div);
				
				tr.appendChild(td);
				this.componentList.appendChild(tr);
			}
			
				var first = null;
				var last = null;
				if(historyPlusDTO.hasOlder) {
                    last = changeSets[changeSets.length - 1];
				}
				if(historyPlusDTO.hasNewer) {
				 	first = changeSets[0];
				}
				 
				this._addPageNavigation(workspace, component, this.versionableDTO.path, this.currentConfigPageSize, first, last);

//				dojox.data.dom.removeChildren(this.pagingControl);
		},
		/**
		 * Display the page navigation links ("Older" and "Newer").
		 * 
		 * @param {Object} workspace
		 * @param {Object} component
		 * @param {Object} n
		 * @param {Object} first
		 * @param {Object} last
		 */
		_addPageNavigation: function(workspace, component, path, n, first, last) {
			
			var div = document.createElement("div");
			div.className = "paging-navigation";
			
			var newerLabel = this.messages.StreamWorkspaceView_58;
			var olderLabel = this.messages.StreamWorkspaceView_59;

			if (first) {
				var anchor = document.createElement("a");
				anchor.rel = "prev";
				anchor.href = ScmUtil.getBrowseElementTabURIWithFirst(workspace.itemId, component.itemId, path, TAB_NAME_HISTORY, first.itemId, n);
				var text = document.createTextNode(newerLabel);
				anchor.appendChild(text);
				div.appendChild(anchor);
			}
			else {
				var span = document.createElement("span");
				span.className = "inactiveLink";
				var text = document.createTextNode(newerLabel);
				span.appendChild(text);
				div.appendChild(span);
			}

			var text = document.createTextNode(this.messages.StreamWorkspaceView_21);
			div.appendChild(text);
			
			if (last) {
				var anchor = document.createElement("a");
				anchor.rel = "next";
				anchor.href = ScmUtil.getBrowseElementTabURIWithLast(workspace.itemId, component.itemId, path, TAB_NAME_HISTORY, last.itemId, n);
				var text = document.createTextNode(olderLabel);
				anchor.appendChild(text);
				div.appendChild(anchor);
			}
			else {
				var span = document.createElement("span");
				span.className = "inactiveLink";
				var text = document.createTextNode(olderLabel);
				span.appendChild(text);
				div.appendChild(span);
			}
			
			this.pagingControl.appendChild(div);
		},
		
		_displayItemHistory: function(historyPlusDTO, versionableDTO) {

			//header
			var tr = document.createElement("tr");
			var th = document.createElement("th");
			th.innerHTML = this.messages.StreamWorkspaceView_29;
			tr.appendChild(th);
			this.tableHeading.appendChild(tr);		
			
			// Expand All | Collapse All
			var span = document.createElement("span");
			dojo.addClass(span, "right");
			th.appendChild(span);
			
			var expd = document.createElement("a");
			expd.appendChild(document.createTextNode(this.messages.FileDiffView_ExpandAll));
			expd.title = this.messages.FileDiffView_ExpandAll_ToolTip;
			var expArgs = {historyItemId: historyPlusDTO.historyDTO.itemId, 
							  workspaceItemId: versionableDTO.workspace.itemId,
							  expand: true};
			dojo.connect(expd, "onclick", dojo.hitch(this, "_toggleAllChangeSets", expArgs));
			span.appendChild(expd);
			span.appendChild(document.createTextNode(" | "));
			
			var coll = document.createElement("a");
			coll.appendChild(document.createTextNode(this.messages.FileDiffView_CollapseAll));
			coll.title = this.messages.FileDiffView_CollapseAll_ToolTip;
			var collArgs = {historyItemId: historyPlusDTO.historyDTO.itemId, 
							  workspaceItemId: versionableDTO.workspace.itemId,
							  expand: false};
			dojo.connect(coll, "onclick", dojo.hitch(this, "_toggleAllChangeSets", collArgs));
			span.appendChild(coll);
			
			var changeSets = historyPlusDTO.historyDTO.changeSets;
			var self = this;
			var count = 0;
			var i = 0;
			dojo.forEach(changeSets, function(changeSet){
				
				var tr= document.createElement("tr");
				self.componentList.appendChild(tr);
				var td = document.createElement("td");
				dojo.addClass(td, "historyView");
				tr.appendChild(td);
				
				var liArgs = {changeSet: changeSet, 
							  historyItemId: historyPlusDTO.historyDTO.itemId,
							  workspaceItemId: versionableDTO.workspace.itemId
							  };
				self._addChangeListItem(liArgs, td);
			});
		},
		
		//args: changeSet, historyItemId, workspaceItemId
		_addChangeListItem: function(args, parentNode) {
			var changeSet = args.changeSet;
			
			var ul = document.createElement("ul");
			dojo.addClass(ul, "changes");
			parentNode.appendChild(ul);
			
			var li = document.createElement("li");
			li.isOpen = false;
			li.changeSetItemId = changeSet.itemId;
			ul.appendChild(li);

			var img = document.createElement("img");
			img.src = ScmUtil.getIconSrc("toggleClosed");
			args.li = li;
			dojo.connect(img, "onclick", dojo.hitch(this, "_toggleExpandableSection", args));
			li.appendChild(img);
			
			var span = document.createElement("span");
			dojo.addClass(span, "comment");
			var hasReason = (changeSet.reasons && changeSet.reasons.length > 0) ? true : false;
			if(hasReason) {
				span.appendChild(ScmUtil.createReasonElement(changeSet.reasons[0])); // TODO: display other associated work items in expanded details section
			}
			var hasComment = (changeSet.comment && changeSet.comment.length > 0) ? true : false;
			if (hasComment) {
				var comment = hasReason ? (" - " + changeSet.comment) : changeSet.comment;
				span.appendChild(document.createTextNode(comment));
			}
			if(!hasReason && !hasComment)
				span.appendChild(document.createTextNode(this.messages.StreamWorkspaceView_33));  // Show <No Comment> if neither reason nor comment exist.
			li.appendChild(span);
			
			if (changeSet.author) {
				var contributorNode = ScmUtil.createContributorElement(changeSet.author);
				dojo.addClass(contributorNode, "comment");
				li.appendChild(contributorNode);
			}
			
			if (changeSet.dateModified) {
				li.appendChild(document.createTextNode(ScmUtil._getDateString(changeSet.dateModified)));
			}
			
			if (changeSet.addedBy && changeSet.dateAdded) {
				li.title = dojo.string.substitute(this.messages.FileDiffView_ChangeAdded_Who_When, [changeSet.addedBy.name, ScmUtil._getDateString(changeSet.dateAdded)]);
			}
			
			var diffDiv = document.createElement("div");
			dojo.addClass(diffDiv, "diff");
			dojo.addClass(diffDiv, "hide");
			li.appendChild(diffDiv);
		},
		
		_toggleExpandableSection: function(args) {
			// toggle the image
			var li = args.li;
			var imgEls = li.getElementsByTagName("img");
			var img = (imgEls && imgEls.length > 0) ? imgEls[0] : null;
			if(img)
				img.src = li.isOpen === true ? ScmUtil.getIconSrc("toggleClosed") : ScmUtil.getIconSrc("toggleOpen");
			
			// toggle the diff section
			var divEls = li.getElementsByTagName("div");
			var div = (divEls && divEls.length > 0) ? divEls[0] : null;
			if(div) {
				if (!li.isOpen) {
					dojo.removeClass(div, "hide");
					// first time display diff, create it
					if(div.childNodes.length == 0) {
						this._retrieveChangeSet(li.changeSetItemId, args.workspaceItemId, dojo.hitch(this, "_displayChange"), {itemId: args.historyItemId, parentNode: div});
					}
				}
				else 
					dojo.addClass(div, "hide");
			}
			// toggle the flag
			li.isOpen = !li.isOpen;			
		},
		
		/*
		 * retrive all the changes in a given change set and display the relavent change.
		 */ 
		_retrieveChangeSet: function(changeSetItemId, workspaceItemId, callback, callbackParam) {			
			PlatformUI.getWorkbench().showMessage(this.messages.ScmPageController_33);				
			var serviceObject = {
				viewChangeSetHandler: function(changeSetDTO){
					
					checkNotNull(changeSetDTO, "changeSetDTO");
					if (changeSetDTO) {
						callback(changeSetDTO, callbackParam);
					}
					else {
						alert(dojo.string.substitute(self.messages.ScmPageController_14, [ componentItemId ]));
					}
					PlatformUI.getWorkbench().hideMessageArea();
				},
				error: function(error){
					if (error.message != "") {
						self._handleError(dojo.string.substitute(self.messages.ScmPageController_19, [ error.message , error.status ]));
					}
					else {
						self._handleError(self.messages.ScmPageController_20);
					}
						
					PlatformUI.getWorkbench().hideMessageArea();
				}
			};
				
			ScmClient.getChangeSet(new ServiceResponseHandler(serviceObject, "viewChangeSetHandler", "error"), {
				workspaceItemId: workspaceItemId,
				changeSetItemId: changeSetItemId
			});
		},
		
		_displayChange: function(changeSetDTO, args) {
			if (changeSetDTO.changes) {
				dojo.forEach(changeSetDTO.changes, function(change){
					// only display the change of the current file(itemId) and not other changes.
					if (change.itemId === args.itemId) {
						var diffView = new com.ibm.team.scm.web.ui.internal.view.browse.FileDiffView();
						diffView.showWidget(change, changeSetDTO);
						args.parentNode.appendChild(diffView.domNode);
					}
				});								
			}
		},
		
		_toggleAllChangeSets: function(args) {
			var self = this;
			dojo.query("li", this.componentList).forEach(
				function(li){
					if(li.isOpen != args.expand) {
						var tsArgs = args;
						tsArgs.li = li;
						self._toggleExpandableSection(tsArgs);
					}
				}, args);
		}
});

})();