/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemComment");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("dojo.date");
dojo.require("jazz.util.html");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SelfSizingTextAreaWidget");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var WorkItemUtil            = com.ibm.team.workitem.web.client.util;
var EditorUtil = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;

/**
 * Top-level work item editor; has a persistent header (defined in this module)
 * and a set of child widgets (mapped in this._childWidgets).
 */
/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemComment",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/WorkItemComment.html"),
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/editor/templates/WorkItemComment.css"),
		// populate the template with data, connect events, and show initial
		// child widget
		isNew: false,
        constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		},	
		postCreate: function() {
			var self = this;
			this._childWidgets = [];
			this.priorTextValue = "";
			this._commentHeader.title = this.messages.titleCommentCollapse;			
			this.commentTextArea = new com.ibm.team.workitem.web.ui.internal.view.layout.SelfSizingTextAreaWidget({parentController: this.parentController});
			dojo.addClass(this.commentTextArea.textarea, "SelfSizingTextAreaComment");
			this.connect(this.commentTextArea.textarea, "onblur",  dojo.hitch(this, this._handleTextBlur));
			this._newCommentArea.appendChild(this.commentTextArea.domNode);
			if (this.comment == null) {
				this._commentHeader.style.display = "none";
				this._oldCommentArea.style.display = "none";
				this.isNew = true;
				if (this.workingCopy) {
					this.workingCopy.addListener({
						path: ["attributes", "comment", "content"],
						event: "onchange",
						listener: this,
						functionName: "_handleWorkingCopyChange"
					});
					var value = this._getAttributeValue();
					if(value) {
						this.setTextValue(value);
						this._setWorkingCopyValue(/* isQuiet */ false , /* notifyEvenIfUnchanged */ true);
					}
				}
				return;
			}
			 	
			this._bindCommentHeader(this.comment);
			this._bindCommentBody(this.comment);
		
		},
		_handleWorkingCopyChange: function(parms) {
			this.setTextValue(parms.attributeValue || "");
		},	
		_getAttributeValue: function() {
			return this.workingCopy.getValue({ path: ["attributes", "comment", "content"] });			
		},
		destroyWidget: function() {
			if(this.commentTextArea) this.commentTextArea.destroy();
			this._stopNewCommentInterval();
			for(var i=0;i< this._childWidgets.length;i++) {
				if(this._childWidgets[i] && this._childWidgets[i].destroy) {
					this._childWidgets[i].destroy();			
				}
				this._childWidgets[i] = null;
			}
			this._childWidgets = [];
			this.destroy();
		},
		_bindCommentHeader: function(comment) {
			// "<span class=\"commentNumber\">" + comment.commentNumber + ".</span>";
			var commentSpan = document.createElement("SPAN");
			commentSpan.className="commentNumber";
			commentSpan.appendChild(document.createTextNode(comment.commentNumber + "."));
			this._commentHeaderSPAN.appendChild(commentSpan);			
		
			var contribName= jazz.util.html.escape(comment.creator.name);
			var contribEmail= jazz.util.html.escape(comment.creator.emailAddress);

// "<span class=\"commentDetails\"><a title='Send an email to the user' href=\"mailto:" + contribEmail + "\">" + contribName + "</a> ";
			var emailSpan = document.createElement("SPAN");
			emailSpan.className="commentDetails";

			var emailSubject;
			if(this.parentController && this.parentController.workingCopy) {
				try {
					emailSubject =  this.parentController.workingCopy.object.attributes.workItemType.label + " " + this.parentController.workingCopy.getId() + ": " + EditorUtil.restoreTags(this.parentController.workingCopy.object.attributes.summary.content);
				}	catch(e) {} // ignore;			
			}    

			var emailHref = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil._getMailto(contribEmail, emailSubject); // subject can be null
			var emailAnchor = document.createElement("A");
			if (comment.creator.locationUri && this.parentController && this.parentController.workingCopy) {
					var resourceLinkArgs= { 
						userId: comment.creator.userId,
						uri: comment.creator.locationUri,
						presentationUri: emailHref,
						label: contribName
					};
					this._childWidgets.push(this.parentController.workingCopy.applyResourceLinkToAnchor(resourceLinkArgs, emailAnchor));
			} else {
				emailAnchor.href = emailHref;
				emailAnchor.appendChild(document.createTextNode(contribName));
			}

			var userIdText = "";
			if(comment.creator.userId) {
	            userIdText = " (@" + comment.creator.userId + ")";
			}		
			emailAnchor.title=  contribName + userIdText;

			emailSpan.appendChild(emailAnchor);
			
			this._commentHeaderSPAN.appendChild(emailSpan);
			
			
//			var commentDate= dojo.date.fromIso8601(comment.creationDate);
			var commentDateStr = this.messages.labelNow;
			if(comment.creationDate) {
				var commentDate= WorkItemUtil.fromISOString(comment.creationDate);
				if(commentDate) {
					commentDateStr= EditorUtil._getDateString(commentDate, true);
				}	
			}
			
			var dateSpan = document.createElement("SPAN");
			dateSpan.className="CommentDate";
			dateSpan.appendChild(document.createTextNode(commentDateStr));
					
			this._commentHeaderSPAN.appendChild(dateSpan);
			
		},

		_bindCommentBody: function(comment) {
            if(!comment || !comment.content || comment.content.length ==0) {
					comment = {
						content: this.messages.labelNoCommentContent
					};
					dojo.addClass(this._commentBodySPAN, "empty-comment");
			};
			var span = EditorUtil.getSafeTextSpan(comment.content, this, comment.links);
			dojox.data.dom.replaceChildren(this._commentBodySPAN,span);
			this._childWidgets = this._childWidgets.concat(EditorUtil.linkAnchorPopUps(span, this.workItem)); // must be in dom
		},
		getAttachmentInfo: function(number) {
			return this.parentController.getAttachmentInfo(number);
		},
		expand: function() {
			this.isCollapsed = false;
			this._commentHeader.title = this.messages.titleCommentCollapse;
			if(this.isNew) {
				this.showNewComment();
//				this.commentTextArea.domNode.style.display="block";
//				this._workItemComment.className = "workItemNewComment";
			} else {
				this._commentBodySPAN.style.display="block";
				this._workItemComment.className = "workItemComment";
			}
		},
		collapse: function() {
			this.isCollapsed = true;
			this._commentHeader.title = this.messages.titleCommentExpand;
			if(this.isNew) {
				this._workItemComment.className = "workItemCommentNewCollapsed";
				this.commentTextArea.domNode.style.display = "none";
			} else {
				this._workItemComment.className = "workItemCommentCollapsed";
				this._commentBodySPAN.style.display="none";
			}
		},
		handleClick: function(e) {
			if(!e || !e.target || this.isNew) return;
			if(e.target.tagName=="A") return;
			if(this.isCollapsed) {
				this.expand();
			} else {
				this.collapse();
			}
		},
		_handleDirtyingEvent: function(e) {

		},
		handleMouseOver: function(e) {
			if(this.isNew) return;
			this._commentHeader.className  = "workItemCommentHeaderMouseOver";
		},
		handleMouseOut: function(e) {
			if(this.isNew) return;
			this._commentHeader.className = "workItemCommentHeader";	
		},
		getTextValue: function() {
			return this.commentTextArea.getValue();
		},
		setTextValue: function(value) {
			if(value == null) value = "";
			this.commentTextArea.setValue(value);
			this.priorTextValue = "";
			this.newCommentValue = "";
		},
		clearTextValue: function() {
			this.commentTextArea.setValue("");
			this.priorTextValue = "";
			this.newCommentValue = ""; 
		},
		showNewComment: function() {
			this._workItemComment.className = "workItemNewComment";
			this._commentHeader.style.display="";
			this._commentHeader.className = "workItemNewCommentHeader";
			this._newCommentArea.style.display="";
			this.commentTextArea.domNode.style.display="";
//		"<span class=\"commentNumber\">&nbsp;</span><span class=\"commentDetails\"><B>New Comment</B></SPAN" ;

//			"<span class=\"newCommentHeader\"><B>New Comment</B></SPAN" ;

			this._commentHeaderSPAN = document.createElement("SPAN");
			this._commentHeaderSPAN.className="newCommentHeader";
			var b= document.createElement("B");
			b.appendChild(document.createTextNode(this.messages.buttonNewComment));
			this._commentHeaderSPAN.appendChild(b);
				
			// force the browser to show the save button
			if (this._saveCmdBottom) {
				try {
					var wasDisabled = this._saveCmdBottom.disabled;
					this._saveCmdBottom.disabled = false;
					this._saveCmdBottom.focus();
					this._saveCmdBottom.disabled = wasDisabled;
				} 
				catch (e) { /* OK, not visible */
				};
							}
			if(this.commentTextArea) {
				try {this.commentTextArea.focus();} catch (e) { /* OK, not visible */};				
			}

			
			// Watch the new comment value for changes
			this.newCommentValue = this.commentTextArea.getValue();
			this._stopNewCommentInterval();
			this.newCommentInterval = setInterval(dojo.hitch(this, this._checkForNewCommentChanges), 200);
			
		},
		hideNewComment: function() {
			this._commentHeader.style.display="none";
			this._newCommentArea.style.display="none";
			this.commentTextArea.domNode.style.display="none";
			this._stopNewCommentInterval();
			
		},
		_checkForNewCommentChanges: function() {
			if(this.newCommentValue != this.commentTextArea.getValue()) {
				this._stopNewCommentInterval();
				if(this.parentWidget){
					var e = {
						srcElement: this.commentTextArea
					};
					this.parentWidget._handleChange(e);
				}
			}
		},
		_stopNewCommentInterval: function() {
			if(this.newCommentInterval) {
				window.clearInterval(this.newCommentInterval);	
			}
		},
		_handleTextBlur: function(e) {
			this._handleTextChange(e, /* notifyEvenIfUnchanged */ true);
		},
		_handleTextChange: function(e, notifyEvenIfUnchanged ) { // key up
			var isQuiet = this.workingCopy.isChanged() && !(notifyEvenIfUnchanged == true);
			this._setWorkingCopyValue(isQuiet, notifyEvenIfUnchanged == true);
		},
		_setWorkingCopyValue: function(isQuiet, notifyEvenIfUnchanged){
			var value = this.getTextValue();
			this.workingCopy.setValue({
				caller: this,
				path: ["attributes", "comment", "content"],
				attributeId: "comment",
				isInitialization: false,
				value: value,
				blankEqualsNull: true,
				isQuiet: isQuiet,
				notifyEvenIfUnchanged: notifyEvenIfUnchanged
			});
		}
	}
);

})();

