/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorHeader");

dojo.require("dojox.data.dom"); // dojo.dom.removeChildren
dojo.require("dojo.date");
dojo.require("dojo.fx");
dojo.require("dojo.i18n");

dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemExtensionRegistry"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil");
dojo.require("com.ibm.team.workitem.web.ui2.internal.action.ActionFactory");
dojo.require("com.ibm.team.workitem.web.ui2.internal.widget.AbstractEditor");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.RequiredPropertyMarkWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.SelectMoveCopyDialog");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.ToolbarWidget");

dojo.require("jazz.util.ResizeNotifier");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

var WorkItemUtil		 	= com.ibm.team.workitem.web.client.util;
var EditorUtil	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var LayoutUtil	= com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil;
var ImageClient				= com.ibm.team.workitem.web.client.internal.ImageClient;
var WorkItemClient 			= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var ImagePool 				= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;

var getAuthenticatedContributor	= com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var isGuest 					= com.ibm.team.repository.web.client.session.isGuest;

var SUBSCRIBERS_ID= "internalSubscriptions";

var COMPATIBLE_TYPES		= [ ["defect", "task", "enhancement"] ];


//Paths to the icons that switch as state changes
var BUTTON_CONTEXT = "com.ibm.team.workitem.web";
var FIND_DUPLICATES_ENABLED=  "ui/internal/view/editor/images/find-duplicates.gif";  // $NON-NLS-1$
var SUBSCRIBE= "ui/internal/view/editor/images/subscribe.gif"; // $NON-NLS-1$
var UNSUBSCRIBE= "ui/internal/view/editor/images/unsubscribe.gif"; // $NON-NLS-1$

var MAX_MESSAGE_LENGTH = 500;

/**
 * Top-level work item editor; has a persistent header (defined in this module)
 * and a set of child widgets (mapped in this._childWidgets).
 */
/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorHeader",
	[com.ibm.team.workitem.web.ui2.internal.widget.AbstractEditor],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/WorkItemEditorHeader.html"),
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/editor/templates/WorkItemEditor.css"),
		_lastSaveTime: null,
		errorHandlingMap: {
			ALERT: function(self, message) {
				alert(message.message + "\n\n" + message.details)
			},
			ADD_DUPLICATE_OF_LINK_ACTION: function(self) {
				self.isValidating = true;
				self.isValidationg = false;
				self.parentController.notifyLinkManagerToAddDuplicateLink();
				self.parentController.showLinksTab();
			},
			SET_RESOLUTION_TO_DUPLICATE_ACTION: function(self) {
				self.isValidating = true;
				self.parentController.showLinksTab();
				self.isValidationg = false;
				alert(self.messages.msgResolutionSet);
			},
			REMOVE_DUPLICATE_OF_LINK_ACTION: function(self) {
				self.isValidating = true;
				self.parentController.showLinksTab();
				self.isValidationg = false;
				self.parentController.notifyLinkManagerToRemoveDuplicateLink();
			},
			SHOW_INFOS_ACTION: function(self) {
				var infos = self._currentInfos;
				if(infos && infos.length> 0) {
					var msg = "";
					for(var i=0;i<infos.length;i++){
						msg += infos[i].summary + ".\n";
					}
					alert(msg);
				}
			}
		},
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

			this.titleSubscribeMe = this.messages.titleSubscribeMe;
			this.titleUnsubscribeMe = this.messages.titleUnsubscribeMe;
			this.titleMoveCopy = this.messages.titleMoveCopy;
			this.titleFindPotentialDuplicates = this.messages.titleFindPotentialDuplicates;
			this.titleCancelWorkItem = this.messages.titleCancelWorkItem;
			this.titleRefreshWorkItem = this.messages.titleRefreshWorkItem;
			this.titleRemoveWorkItem = this.messages.titleRemoveWorkItem;
			this.buttonCancel = this.messages.buttonCancel;
			this.buttonSave = this.messages.buttonSave;
			this.progressMessage = this.messages.busyLoading;
			this.labelSummarySafari = this.messages.labelSummary;		
		
		},
	
		
		// populate the template with data, connect events, and show initial
		// child widget
		postCreate: function() {
			checkNotNull(this.parentController, "this.parentController");
			checkNotNull(this.editorUtil, "this.editorUtil");
			this.isHeader = true;
			this._childWidgets = [];
			this.PROPERTIES= {	};

			if(dojo.isSafari) {
				var reqMark = new com.ibm.team.workitem.web.ui.internal.view.layout.RequiredPropertyMarkWidget(
						{workingCopy: this.parentController.workingCopy,
						 attributeId: "summary"}
					);
				this._childWidgets.push(reqMark);
				this.summaryRequiredMarker.appendChild(reqMark.domNode);
				// Defect 66867 Truncated summary (pl locale this.labelSummary = "Podsumowanie: ";)
				if (this.labelSummarySafari.length > 12) {
					this.summaryLabelDIV.className = "SummaryLabelVeryLong";
				}
				else 
					if (this.labelSummarySafari.length > 9) {
						this.summaryLabelDIV.className = "SummaryLabelLong";
					}
				this.summary = this.summarySafari;
				this.headerSummary = this.headerSummarySafari;
			} else {
				//<div class="SummaryLabel" dojoAttachPoint="summaryLabelDIV"><span>${labelSummary}</span><span dojoAttachPoint="summaryRequiredMarker" class="RequiredMarker">*</span></div>
				var nameValueLayout = new com.ibm.team.workitem.web.ui.internal.view.layout.NameValueLayout({
						attributeId: "summary",
						workingCopy: this.parentController.workingCopy, 
						hasRequiredMarker: true });
				nameValueLayout.addRow( { labelText: this.messages.labelSummary, valueNode: this.summaryContainer });
				this.headerSummary.appendChild(nameValueLayout.domNode);
				this._childWidgets.push(nameValueLayout);
			}
			
			this._isInitialized = false;
			this._updateTypeIcon();
			
			this.layoutHeader();
			this._resizeEvent = jazz.util.ResizeNotifier.register({context: this, callback: this._onResize});

			this._handles = [ dojo.subscribe(com.ibm.team.workitem.web.ui.internal.page.EventTopics.WORKITEM_CHANGED, dojo.hitch(this, this._handleWorkItemChanged)) ];

			this._toolbar = new com.ibm.team.workitem.web.ui.internal.view.layout.ToolbarWidget({});
			this._childWidgets.push(this._toolbar);
			this._simpleToolbarContainer.appendChild(this._toolbar.domNode);

			var extensionActions = com.ibm.team.workitem.web.client.internal.WorkItemExtensionRegistry.getContributedEditorToolbarActions();
			if(extensionActions.length > 0) {
				for(var i=0;i<extensionActions.length; i++) {
					var actionSpec = extensionActions[i];
					if(actionSpec.action) {
						var action = com.ibm.team.workitem.web.ui2.internal.action.ActionFactory.getAction({actionSpec: actionSpec, workingCopy: this.workingCopy});
						if(!action) {
							console.debug("Missing action: " + actionSpec.action);
							continue;
						}
						this._toolbar.addTwoStateButton(/*context*/ action, action._doRun, {imgSrc: action.getIconUri(), label: actionSpec.label, alt: actionSpec.alt});
					}
				}
			
				// Add a separator between the extensions and default actions
				this._toolbar.addSeparator();
			}

			this._findDuplicatesButton =  this._toolbar.addTwoStateButton(this, this._handleFindDuplicatesClick,
				 {isEnabled: true, 
				 	imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, FIND_DUPLICATES_ENABLED), 
				 	label: this.titleFindPotentialDuplicates});

			this._subscribeSelfButton = this._toolbar.addTwoStateButton(this, this._handleSubscribeSelfClick, 
				{isEnabled: true, 
					imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, SUBSCRIBE),
					label: this.titleSubscribeMe});

			this._moveCopyButton = this._toolbar.addTwoStateButton(this, this._handleMoveCopyClick, 
				{isEnabled: true, 
				 	isVisible: false,
					imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, "ui/internal/view/editor/images/move-copy.gif"),
					label: this.titleMoveCopy});

			this._refreshButton = this._toolbar.addTwoStateButton(this, this._handleRefreshClick, 
				{isEnabled: true, 
				 	isVisible: true,
					imgSrc: dojo.moduleUrl(BUTTON_CONTEXT, "ui/internal/view/editor/images/refresh.gif"),
					label: this.titleRefreshWorkItem});

		},
		_handleWorkItemChanged: function(workingCopy) {
			if(workingCopy === this.parentController.workingCopy) {
				this._toolbar.handleContextChanged({ workingCopy: workingCopy});
				this.disableSave(/*isDisabled*/!workingCopy.isChanged());
			} 
		},
		/**
		 * Lookup with image in the current plug-in.
		 */
		_lookupImg: function(imagePath) {
			return  dojo.moduleUrl("com.ibm.team.workitem.web",imagePath); ;  //$NON-NLS-1$
		},
		destroy: function() {
			jazz.util.ResizeNotifier.unregister(this._resizeEvent);
			for(var i=0;i<this._handles.length;i++) {
				dojo.unsubscribe(this._handles[i]);
			}
			// this._childWidgets are destroyed by inherited AbstractEditor protocol
			this.inherited(arguments);
		},
		_onResize: function() {
			var width = dojo.coords(this.summary).w;
			if(width> 285) {
				dojo.removeClass(this.headerSummary, "SummaryWrapperWide");
				dojo.addClass(this.headerSummary, "SummaryWrapper");
			} else {
				dojo.removeClass(this.headerSummary, "SummaryWrapper");
				dojo.addClass(this.headerSummary, "SummaryWrapperWide");
			}
		},
		layoutHeader: function() {
			
			this.workingCopy = this.parentController.workingCopy;
			if(!this.workingCopy) return; // not yet initialized

			if (this.workingCopy.workItemSpec!=null && this.workingCopy.workItemSpec.presentationProps != null) {
				var listDiv = document.createElement("div");
				var pages = this.workingCopy.workItemSpec.presentationProps.pages;
				if (!pages || pages.length == 0) 
					return;
				var headerPage;
				for (var i = 0; i < pages.length; i++) {
					if (pages[i].layout == "internalHeader") { // $NON-NLS-1
						headerPage = pages[i];
						break;
					}
				}
				dojox.data.dom.removeChildren(this.headerLeft);
				dojox.data.dom.removeChildren(this.headerRight);
				this.headerSummary.style.display="none";
				if (headerPage && headerPage.sections && headerPage.sections.length > 0) {
					var presentations = headerPage.sections[0].presentations;
					var isLeft = true;
					for (var i = 0; i < presentations.length; i++) {
						if(presentations[i].widget.indexOf("SeparatorPart") != -1)
							continue;
						if(presentations[i].attributeId == "summary") {
							this.headerSummary.style.display="";
							isLeft = false;
							continue;
						}
						var widgetDiv = document.createElement("div");
						this._layoutPresentation(presentations[i], widgetDiv, null, {webHeight: "1.2em"} );
						var targetDiv = isLeft ? this.headerLeft : this.headerRight;
						targetDiv.appendChild(widgetDiv);
					}
				} 
			}
		},
		_layoutPresentation: function(pres, attrDiv, attrClass, modifiers){
			var valueDiv = document.createElement("div");
			var values = this._getPresentationValues(pres);
			var possibleValues = this._getPresentationPossibleValues(pres);
			var widgetDiv = document.createElement("div");
			var widget = this.getWidget(this.workingCopy.object, pres, widgetDiv, values, possibleValues, modifiers);
		
			var properties = LayoutUtil.getPropertyMap(pres);
			attrDiv.className = "attr-container-narrow";
			var labelDiv = document.createElement("div");
			if(properties.labelVisible != "false" && dojo.isString(pres.attributeLabel)) {
				attrDiv.className = "attr-container";
				labelDiv.appendChild(document.createTextNode(dojo.string.substitute(this.messages.labelValue, [pres.attributeLabel])));
			}
			labelDiv.className = "attr-label";
			var requiredMarkerSpan = document.createElement("SPAN");
			var isRequired = this._isRequired(pres.attributeId);
			
			requiredMarkerSpan.className = "RequiredMarker";
			requiredMarkerSpan.innerHTML = this.messages.labelRequired; // "* " innerHTML OK 
			requiredMarkerSpan.title = this.messages.titleRequiredMarker;
			if (isRequired) {
				requiredMarkerSpan.style.display = "inline";
			}
			else {
				requiredMarkerSpan.style.display = "none";
			}
			labelDiv.appendChild(requiredMarkerSpan);
			
			attrDiv.appendChild(labelDiv);
			
			var width;
			if(widget.getMinimumWidth) {
			 	width = widget.getMinimumWidth();				
			}

			valueDiv.className = attrClass || "attr-value";
			valueDiv.appendChild(widget.domNode);
			if(width) {
				attrDiv.style.width = width;
			}
			attrDiv.appendChild(valueDiv);
		},
	_isRequired: function(attributeId){
		var isRequired = false;
		if (this.workingCopy.workItemSpec && this.workingCopy.workItemSpec.editProps && this.workingCopy.workItemSpec.editProps.requiredProperties) {
			for (var i = 0; i < this.workingCopy.workItemSpec.editProps.requiredProperties.length; i++) {
				if (this.workingCopy.workItemSpec.editProps.requiredProperties[i] == attributeId) {
					isRequired = true;
					break;
				}
			}
		}
		return isRequired;
	},
	_getPresentationValues: function (pres) {
		var attrId = pres.attributeId;
		var value = {};
		if(this.workingCopy.storedObject && this.workingCopy.storedObject.attributes) {
			value = this.workingCopy.storedObject.attributes[attrId];
		}
		return [ value ];
	},
	_getPresentationPossibleValues: function (pres) {
		var attrId = pres.attributeId;
		var allValues = [];
		if(this.workingCopy && this.workingCopy.workItemSpec && this.workingCopy.workItemSpec.editProps && this.workingCopy.workItemSpec.editProps.allValues) {
				allValues = this.workingCopy.workItemSpec.editProps.allValues;
		}
		
		for(var i=0;i<allValues.length;i++) {
			if(allValues[i].attributeName == pres.attributeId) {
				return allValues[i].uiItems;
			}
		}
		
		return [  ];
	},
	   _handleDirtyingEvent: function() {
	   		this.parentController.markDirty();
	   },
		_handleWorkingCopySummaryChange: function(parms) {
			this.summary.value= parms.attributeValue;
			this.parentController.markDirty();
		},
		_getAttributeValue: function() {
			return this.parentController.workingCopy.getValue({ path: ["attributes", "summary", "content"] });			
		},
		_setAttributeValue: function(value, isInit, isQuiet, notifyEvenIfUnchanged){
			this.parentController.workingCopy.setValue({
				caller: this,
				path: ["attributes", "summary", "content"],
				savedAttributePath: ["attributes", "summary", "content"],
				attributeId: "summary",
				isInitialization: isInit,
				forceNotify: notifyEvenIfUnchanged,
				isQuiet: isQuiet,
				value: value
			});
		},
		markBusy: function(text) {
			if(!text) text = this.messages.busyWorking;
			// show the loading indicator
			this._showTypeAndId(text, "", true);				

		},
		focus: function(doResize) {
			if(doResize != false) this._onResize();
			if(this.summary) {
				setTimeout(dojo.hitch(this, function() { this.summary.focus()}), 0);
			}
		},
		reset: function(isTypeChange) {			
			// unbind work item data (this.typeName is cleared by this.markBusy)
			if (isGuest()) {
				dojo.addClass(this.summary, "GuestField");
				this._subscribeSelfButton.setVisible(false);
			}
			
			dojox.data.dom.removeChildren(this._idNumber);
			this._idNumber.isDirty = false;
			
			this._findDuplicatesButton.setEnabled(false);
	
			this._hasStickyMessage = false;
			this.isValidating = false;
			this._isAutomaticallySettingResolutionToDuplicate = false;
			this.clearValidationMessage();
			for (var id in this.PROPERTIES) {
				this.editorUtil._clearRequired(this.PROPERTIES[id].requiredMarker);
			}
			
			var isChanged = false;
			if(this.parentController.workingCopy) {
				isChanged=this.parentController.workingCopy.isChanged();				
			} 
			
			this.disableSave(/*isDisabled*/!isChanged);
			// reset internal state
			this.summary.__jazzChanged= false;
			
			if(isTypeChange) {
				this.summary.focus();
			}
		},

		setHeaderToolbarState: function(isNew) {
			if(isNew) {
				this.cancelCmd.style.display= "";
				this._subscribeSelfButton.setVisible(false);
				this._refreshButton.setVisible(false);
			} else {
				this.cancelCmd.style.display= "none";
				this._subscribeSelfButton.setVisible(true);
				this._refreshButton.setVisible(true);
			}
			
		},
		_showTypeAndId: function(typeName, idLabel, isLoading) {
				
			    if (isLoading) {
					dojox.data.dom.removeChildren(this.progressMsg);
					this.progressMsg.appendChild(document.createTextNode(typeName));
					this.typeIcon.style.visibility = "hidden";
					this.typeNameNode.style.visibility = "hidden";
					this.progressNode.style.visibility = "visible";
				}
				else {
					dojo.removeClass(this.typeIcon, "LoadingIcon");
					dojox.data.dom.removeChildren(this._idNumber);
					dojox.data.dom.removeChildren(this.typeName);
					this.typeName.appendChild(document.createTextNode(typeName));
					if (idLabel && this._idNumber.isDirty) {
						idLabel = idLabel.replace(/ \*/, "") + " *";
					}
					this._idNumber.appendChild(document.createTextNode(idLabel));
					this.disableSave(this.saveCmd.disabled);
					
					this.typeIcon.style.visibility = "visible";
					this.typeNameNode.style.visibility = "visible";
					this.progressNode.style.visibility = "hidden";
				}

		},		

		// bind work item data to the editor header (each sub-tab is responsible
		// for binding its own data)
		bindWorkItem: function(workItem, retainedState, isTypeChange, newTypeId) {
			this.parentController.workingCopy.removeAllListeners({listener:this});
			this.parentController.workingCopy.addListener({
				path: ["attributes", "summary", "content"],
				event: "onchange",
				listener: this,
				functionName: "_handleWorkingCopySummaryChange"
			});			

			// save a reference to the work item
			this._workItem= workItem;
			this._hasStickyMessage = false;
			this.isValidating = false;
			this._isAutomaticallySettingResolutionToDuplicate = false;
			this.clearValidationMessage();
			
			if(!isTypeChange ){
				this._updatedTypeName = null;
				this._updatedTypeIconSrc = null;
			}
			
			// enable or disable controls based on work item state
			this.updateSubscribeSelf();

			this._disableRefresh(false); // gets enabled as "reset" for new work items			
			
			this.showWorkItemIdAndType(isTypeChange);
			
			var isCompatibleTypeChange = true;	
			this.summary = this.summary ? this.summary : {};						
			if(isTypeChange){
				var isCompatibleTypeChange = false;
				if (retainedState.summary) {
					this.summary.__jazzChanged = true;
					this.summary.value = retainedState.summary
				} else {
					this.summary.value = EditorUtil.restoreTags(this._getAttributeValue());
				}
				
				/*
				 * If it is a type change, the actions and resolutions
				 * can potentially be no longer valid.
				 * 
				 * We will assume that changes among defect,task and enhancement 
				 * are compatible and leave the actions and resolution enabled,
				 * but for changes away from or into these, we will disable
				 * the actions and resolution.
				 */
				var priorType = this._workItem.attributes.workItemType.id;
		
				for(var i=0;i<COMPATIBLE_TYPES.length;i++) {
					var typeSet = COMPATIBLE_TYPES[i];
					var found = false;
					for(var j=0;j<typeSet.length;j++) {
						if(typeSet[j]==priorType){
							found = true;
							break;
						}
					}
					if(found) {
						for(var j=0;j<typeSet.length;j++) {
							if(typeSet[j]==newTypeId){
								isCompatibleTypeChange = true;
							}
						}
					}
					if(isCompatibleTypeChange) break;					
				};				
			} else {
				var isNew = this.parentController.workingCopy.isNewWorkItem();
				if (this._isInitialized || !isNew || this.summary.value == "") {
					var content = this.parentController.workingCopy.getValue({path: "attributes.summary.content"}) || "";
					this.summary.value = EditorUtil.restoreTags(content);
				} else {
					if(isNew && this.summary.value != ""){
						this.markDirty();
					}
				}
			}
			this._isInitialized = true;

			return isCompatibleTypeChange;
		},
		endOperation: function() {
			this.showWorkItemIdAndType();
		},
		showWorkItemIdAndType: function(isTypeChange) {
			// bind work item data to UI controls
			var typeName = this._workItem.attributes.workItemType.label;
			if(isTypeChange && this._updatedTypeName) {
				typeName = this._updatedTypeName;
				this.layoutHeader();
			}
			var idLabel = this.parentController.workingCopy.getIdLabel();

			this._showTypeAndId(typeName, idLabel, /* isLoading */ false);			
	},
		
		bindOptionalValues: function(valMap, retainedState, isTypeChange, isCompatibleTypeChange) {

		},
		
		bindRequiredProperties: function(requiredProps) {
			for (var i=0; i<requiredProps.length; i++) {
				var requiredProp= this.PROPERTIES[requiredProps[i]];
				if (requiredProp) {
					this.editorUtil._markRequired(requiredProp.requiredMarker);
				}
			}
		},
		
		markFullyLoaded: function() {
			
			// If the type was updated, use that icon instead of the work item's
			if(this._updatedTypeIconSrc) {
				this._updateTypeIcon(this._updatedTypeIconSrc);
				return;
			}
			if (this._workItem.attributes.workItemType.iconUrl) {
				this._updateTypeIcon(ImageClient.getImageUrl(this._workItem.attributes.workItemType.iconUrl));
			} else {
				this._updateTypeIcon(/* hide it */ null);
			}
			this._enableFindDuplicatesIcon();
		},
		
		//
		// 'public' methods available to the controller
		//
		
		disableSave: function(isDisabled) {
			isDisabled = isDisabled || this.parentController.workingCopy.isInOperation();
			this.saveCmd.disabled= isDisabled;
			this.parentController.disableSave(isDisabled);
			this._enableFindDuplicatesIcon();
			if (!isDisabled) {
				// Add the isDirty asterisk if it is not already there 
				if (this._idNumber.innerHTML && this._idNumber.innerHTML.indexOf("*") == -1) {
					this._idNumber.isDirty = true;
					this._idNumber.innerHTML = this._idNumber.innerHTML + " *"; /* innerHTML OK */
				}
			} else {
					this._idNumber.isDirty = false;
					this._idNumber.innerHTML = this._idNumber.innerHTML.replace(/ \*/, "");
			}
		},
				
		// implementation method for only the editor's form elements
		getFormElements: function() {
			return {};
		},
		
		markDirty: function() {
			// Enable top and bottom save button
			this.disableSave(/*isDisabled*/false);
		},
		
		/**
		 * Updates state of the "Subscribe Me" control to either add, remove or disabled
		 * based on what is currently allowed.
		 */
		updateSubscribeSelf: function() {
			if(!this._workItem) return;
			var loggedInContributor= getAuthenticatedContributor();

			var isEnabled= !isGuest() && !this.parentController.isNewWorkItem(); 

			if (isEnabled) {
				var isSubscriber= this.parentController.workingCopy.includesItem({
					item: loggedInContributor,
					attributeId: SUBSCRIBERS_ID
				});
				this._subscribeSelfButton.setEnabled(true);
			}
			else {
				this._subscribeSelfButton.setEnabled(false);
			}
			if (isEnabled && isSubscriber) {
				this._subscribeSelfButton.isAdd = false;
				this._subscribeSelfButton.updateButton({
					imgSrc: this._lookupImg(UNSUBSCRIBE),
					label: this.titleUnsubscribeMe
				})
			}
			else {
				this._subscribeSelfButton.isAdd = true;
				this._subscribeSelfButton.updateButton({
					imgSrc: this._lookupImg(SUBSCRIBE),
					label: this.titleSubscribeMe
				})
			}
		},

		// update the type (e.g. defect or task) based on the type code (type change)
		updateType: function(name, imgSrc, isInitialization) {
			if(isInitialization && this._updatedTypeName) return;
			imgSrc = imgSrc ||ImagePool.PIXEL.uri;
			if(!isInitialization) {
				this._updatedTypeName = name;
				this._updatedTypeIconSrc = imgSrc;
			}
			this.markBusy(this.messages.busyLoading);
		},
		
		_updateTypeIcon: function(imgSrc) {
			if(dojo.hasClass(this.typeIcon, "validationMessageErrorIcon")){
				this.typeIcon.style.visibility="visible"
				return;
			}

			if(!imgSrc) {
				this.typeIcon.style.visibility="hidden"
			}
			
			this.typeIconSrc = imgSrc;
			this.typeIcon.src = imgSrc;
			// don't show the icon if an operation is showing progress
			if(imgSrc && this.progressNode.style.visibility != "visible" ) {
				this.typeIcon.style.visibility="visible"
			}
		},					
		
		//
		// ui initialization methods
		//
		
		_disableRefresh: function(isDisabled) {
			this._refreshButton.setEnabled(isDisabled ? false : true);
		},
		
		//
		// event handlers
		//
		// focus lost causes the propagation (e.g., to 
		_handleSummaryKeyUp: function() {
			this._checkSummaryValue();
		},		
		_handleSummaryBlur: function(e) {
			this._checkSummaryValue();
		},
		_checkSummaryValue: function(e) {
			var oldValue = this.parentController.workingCopy.getValue({path: "attributes.summary.content"});
			if(oldValue == this.summary.value) return;
			var oldValueWithoutTags = EditorUtil.restoreTags(oldValue);
			if(oldValueWithoutTags == this.summary.value) return;
			this._handleSummaryChange(e, /* notifyEvenIfUnchanged */ true);
		},
		// keystrokes update the value, but do not propagate
		_handleSummaryChange: function(e, notifyEvenIfUnchanged) {
			var isQuiet = this.parentController.workingCopy.isChanged() && !(notifyEvenIfUnchanged == true);
			var value = this.summary.value;
			if(this.parentController.workingCopy && this.parentController.workingCopy.object && this.parentController.workingCopy.object.attributes && this.parentController.workingCopy.object.attributes.summary) {
				if (value != this.parentController.workingCopy.object.attributes.summary.content) {
					this._setAttributeValue(this.summary.value, null, isQuiet, notifyEvenIfUnchanged);
					this.parentController.markDirty();
				}		
			}
//			if(!notifyEvenIfUnchanged && this.summary && this.summary.value == value) return; // not changed
		},
		_enableFindDuplicatesIcon: function() {
			if(!this._workItem) return;
			if(this._findDuplicatesButton.isEnabled) return; // already enabled
			var isEnabled = !this.parentController.workingCopy.isNewWorkItem();
			if (!isEnabled) {
				if (this.summary && this.summary.value.length > 0) {
					isEnabled = true;
				}
				else {
					var descWidget = this.parentController.findObjectWithProperty({
						property: "isDescriptionManager"
					});
					if (descWidget && descWidget.getDescriptionValue) {
						if (descWidget.getDescriptionValue().length > 0) {
							isEnabled = true;
						}
					}
				}
			}
			
			this._findDuplicatesButton.setEnabled(isEnabled);
		},
 		_handleMoveCopyClick: function(e) {
			dojo.stopEvent(e);
			if(!this._moveCopyButton.isEnabled) return;
			this._openMoveCopyDialog();
		},
		displayMoveCopy: function(isEnabled) {
			this._moveCopyButton.setVisible(isEnabled);
		},
		_openMoveCopyDialog: function() {
			var self= this;
			var params= {
				projectArea: this.parentController.workingCopy.getValue({path: "attributes.projectArea"}),
				parentController: this.parentController,
				parentElement: this.domNode,
				onOk: function(parms) {
					self._moveCopyOK(parms);
				},
				onClose: function() {
					self.parentController.focus();
				}
			};
			new com.ibm.team.workitem.web.ui.internal.view.editor.SelectMoveCopyDialog(params);

		},
		_moveCopyOK: function(parms /* isMove, projectAreaItemId */) {
			 var self = this;
				parms.onSuccess= function(args) {
					self._showInfosMessages(args);
				};
				parms.onError = function(args) {
					console.debug("move/copy error"); // $NON-NLS-1
					// failures have already been logged
					// TODO: test this path
				}
	    	this.clearValidationMessage(true);
			this.workingCopy.moveOrCopyToProjectArea(parms);
		},
		_showInfosMessages: function(parms) {
			this._currentInfos = parms.infos || this.parentController.workingCopy.infos;
			if(this._currentInfos && this._currentInfos.length != 0) {
				this._showInfos(this._currentInfos);
			}
			if(parms.isMove) {
				this.parentController.markDirty();
			}
		},
		_handleFindDuplicatesClick: function(e) {
			dojo.stopEvent(e);
			if(!this._findDuplicatesButton.isEnabled) return;
			if(!this._workItem) return;
			if(this.parentController.workingCopy.isNewWorkItem()) { // new
				var summary = dojo.trim(this.summary.value);
				var content = ""; 
				
				var descWidget = this.parentController.findObjectWithProperty({
					property: "isDescriptionManager"
				});
				if(descWidget && descWidget.getDescriptionValue) {
					content=dojo.trim(descWidget.getDescriptionValue());
				}
				this.parentController.pageController.openSimilarityQuery( {
					summary: summary,
					content: content,
					projectAreaItemId: this.parentController.workingCopy.getProjectAreaItemId()
				});			
				
			} else {
				this.parentController.pageController.openSimilarityQuery( {
					itemId:  this._workItem.itemId,
					forceRefresh: true,
					projectAreaItemId: this.parentController.workingCopy.getProjectAreaItemId()
				});
			}			
		},
		_handleSubscribeSelfClick: function(e) {
			e.preventDefault();
			if(!this._subscribeSelfButton.isEnabled) return;

			this.parentController.markDirty();
			if (this._subscribeSelfButton.isAdd) {
				this.parentController.workingCopy.addItems( {
					items: [getAuthenticatedContributor()],
					attributeId: SUBSCRIBERS_ID
				});
			}
			else {
				this.parentController.workingCopy.deleteItems({
					items: [getAuthenticatedContributor()],
					attributeId: SUBSCRIBERS_ID
				});
			}
			this.updateSubscribeSelf();
		},
		
		/*
		 * Respond to the user clicking the 'Save' control.
		 */
		_handleSaveClick: function(e) {
			this._lastSaveTime = new Date();
			this.parentController.save();
		},
		
		/*
		 * Respond to the user clicking the 'Refresh' control.
		 */
		_handleRefreshClick: function(e) {
			e.preventDefault();
			if(!this._refreshButton.isEnabled) return;
			// since we're going to update the work item
			this._lastSaveTime = null;
			this.parentController.refresh();
		},
		
		/*
		 * Respond to the user clicking the 'Cancel' button
		 */
		_handleCancelClick: function(e) {
			e.preventDefault();
			if(!this.saveCmd.disabled) {
				if(!confirm(this.messages.promptCancelWorkItemEdit)) {
					return;
				}
			}
			this.parentController.pageController.removeEditor(this.parentController.workingCopy.getId(), true);
			this.parentController.destroy();
			this.parentController._sendChangedEvent("Closed", true);
		},
				
		clearValidationMessage: function(force) {
			if(!force && this._skipNextClear ) {
				this._skipNextClear = false;
				return;
			} 
			this._skipNextClear = false; // forced
			if(!this._hasStickyMessage) {
				this.validationMessage.__jazzMessage = -1;
				dojox.data.dom.removeChildren(this.validationMessage);
			} 
			if(this.typeIconSrc) {
				this.typeIcon.src = this.typeIconSrc;
			}
			dojo.removeClass(this.typeIcon, "validationMessageErrorIcon");
			this.validationMessageSpan.className = "validationMessageEmpty";
		
		},
		hasDuplicateLink: function() {
			 return this.parentController.hasDuplicateOfLink();
		},
		_showInfos: function(infos) {
			if(!infos || infos.length == 0 ) return;
			    if(infos.length > 1) {
			    	var msg = dojo.string.substitute(this.messages.msgInfosMore, [ infos[0].summary, infos.length-1]);					
				} else {
					var msg = infos[0].summary;
				}

				this.setHeaderMessage({
								message: msg,
								action: "SHOW_INFOS_ACTION",
								isSticky: false,
								isError: false
							})
		},
		setHeaderMessage: function(/* message, action, isSticky, severity */ message) {
			this.clearValidationMessage();
			if(!message) {
				return;
			}
			
			dojo.addClass(this.typeIcon, "validationMessageErrorIcon");
			
			if(!message.message || message.message.length == 0) {
				message.message = this.messages.errorUnhelpful;
			} else {
				if(message.message.lastIndexOf(".") != (message.message.length-1)) {
					message.message += ".";
				}
			}
            
            if (!message.details) {
                message.details = "";
            }
            else {
                if (message.details.lastIndexOf(".") != (message.details.length - 1)) {
                    message.details += ".";
                }
            }
            var messageText;
            if (message.message.length > MAX_MESSAGE_LENGTH) {
                messageText = message.message.substring(0, MAX_MESSAGE_LENGTH) + "...";
            }
            else {
            	if(message.showOnlyMessage == true) {
            		messageText = message.message
            	} else {
                	messageText = message.message + " " + message.details;            		
            	}
                if (messageText.length > MAX_MESSAGE_LENGTH) {
                    messageText = messageText.substring(0, MAX_MESSAGE_LENGTH) + "...";
                }
            }
            messageText = messageText.replace(/\.\.\.\./, "...");
            
            this._hasStickyMessage = message.isSticky == true;
            this.validationMessage.__jazzMessage = message;
            if (message.severity==8) {
				this.typeIcon.src = ImagePool.ERROR;
                this.validationMessageSpan.className = "validationMessageError";
            }
            else {
				if (message.severity==4) {
					this.typeIcon.src = ImagePool.WARNING;
					this.validationMessageSpan.className = "validationMessageWarning";
					this._skipNextClear = true; // hack... we know there is an extra clear on setting resolutions
				} else {
					this.typeIcon.src = ImagePool.INFO;
					this.validationMessageSpan.className = "validationMessageInfo";
					this._skipNextClear = true; // hack... we know there is an extra clear on setting resolutions					
				}
            }
            this.validationMessage.title = message.details || message.message;
            dojox.data.dom.replaceChildren(this.validationMessage, document.createTextNode(messageText));
			this.focus();
			window.scrollTo(0,0); // show message
		},
		updateValidationMessage: function(wasResolutionChanged) {
			// nothing to do now
		},
		_handleHeaderMessageClick: function(e) {
			e.preventDefault();
			if(!e || !e.target || !e.target.__jazzMessage) return;

			var actionFunc = this.errorHandlingMap[e.target.__jazzMessage.action];
			
			if(!actionFunc ) return;
			
			actionFunc(this,e.target.__jazzMessage );
 
		},
		/*
		 * Update the last saved or loaded timestamp
		 */
		updateRefreshTimestamp: function(isNew) {
			if(isNew) {
				this._refreshButton.updateButton({label: this.messages.titleResetWorkItem})
				return;
			} else {
				this._refreshButton.updateButton({label:  this.messages.titleRefreshWorkItem})
			}
					
			var now, prefix;
			if(this._lastSaveTime) {
				prefix = this.messages.labelSaved;
				now = this._lastSaveTime;
			} else {
				prefix = this.messages.labelLoaded;
				now = new Date();
			}
			try{
				var timeString = this.editorUtil._getDateString(now,true);
				var message;
				message = dojo.string.substitute(prefix, [ timeString ]);
				
				dojox.data.dom.replaceChildren(this.lastRefreshText, document.createTextNode(message));

				this.lastRefreshText.style.display="block";

			} catch(e) {
				// if there is a problem, just use the default now date
				this.lastRefreshText.style.display="none";
				console.debug(dojo.string.substitute(this.messages.errorInUpdateRefreshTimestamp, [ (""+e) , message ]));
			}
		}
	}
);

})();
