/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.ui.page.SubmitMenuManager");

dojo.require("dojo.i18n");
dojo.require("com.ibm.rational.cm.web.ui.page.RecordTypeManager");

dojo.requireLocalization("com.ibm.rational.cm.web.ui", "cqPageController");

(function(){

var maxSubmitMenu = 10;
var messages, _submitDlg;
var cq = com.ibm.rational.cm.web;

dojo.declare(
	"com.ibm.rational.cm.web.ui.page.SubmitMenuManager",
	null,
{
	defaultRecordType: null,
	
	constructor: function(){
		messages = dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "cqPageController");
	},
	
	initialize: function(/* jazz.ui.Button */ button, connect, /* Object */ obj){
		this._button = button;
		this._connect = connect;
		this.defaultRecordType = obj;
	},
	
	populateSubmitMenu: function(){
		var self = this;
		com.ibm.rational.cm.web.ui.page.recTypeManager.getSubmitTypes({
			onLoad: function(submitTypes){
				// first disconnect
				var button = self._button;
				button.disconnect(self._connect);

				// first keep original child menu items (loading menu)
				var menu = button._menu;
				var prevMenuItems;
				if(menu){
					prevMenuItems = menu.getChildren();
				}
				
				// create new menu items
				var args = [ self.defaultRecordType ];
				for(var i=0, num=Math.min(maxSubmitMenu,submitTypes.length); i<num; i++){
					var type = submitTypes[i];
					if(!type.LocationId || !type.DisplayName || type.LocationId == "undefined" ||
						type.LocationId == self.defaultRecordType.id) continue;
					args.push({
						label: type.DisplayName,
						url: "#action="+cq.ACTION_PREFIX+"newRecord&type=" + encodeURIComponent(type.LocationId) + "&title=" + encodeURIComponent(type.DisplayName),
						id: type.LocationId
					});
				}

				// Prepare for dialog if too many Submit items ("More..." menu)
				if(submitTypes.length > maxSubmitMenu){
					var candidates = dojo.map(submitTypes, function(d){
						return {
							name: d.DisplayName,
							value: d.LocationId
						};
					});
					candidates.sort(function(a,b){
						if(a.name == b.name){
							return 0;
						}else if(a.name > b.name){
							return 1;
						}else{
							return -1;
						}
					});

					args.push({
						label: messages.More,
						url: "javascript:void(0);",
						callback: function(){
							self.showNewRecordDialog(candidates);
						},
						id: "moreSubmitTypes"
					});
				}
					
				// create create record menus
				button.populateDropDown(args);
				if(button.makeUrlsUnique){
					button.makeUrlsUnique();
				}
				dojo.forEach(args, function(item, idx){
					var menuItem;
					if(item.callback){
						var id = item.id;
						menuItem = new dijit.MenuItem({
							label: item.label,
							onClick: function(event){
								this.dropDownValue.callback(event, this.dropDownValue);
							},
							dropDownValue: item
						})
					}else{
						menuItem = new dijit.MenuItem({
							label: item.label,
							onClick: function(){
								location.href = this.url
							}
						})
						menuItem.url = item.url;	
					}
					menu.addChild(menuItem);
					if(item.iconUrl) {
						var img = new Image();
						img.src = item.iconUrl;
						menuItem.iconNode.appendChild(img);	
					}
					item.menuItem = menuItem;
					
					// This is a workaround for what seems to be dojo Menu bug
					// without this, error occurs when putting mouse over loading menu, and hovering over other menus after menus are loaded
					// this is caused by Menu trying to set focus to savedFocus in _openMyself
					if(idx == 1){
						menu.focusChild(menuItem)
					}
				});
				
				// remove loading menu item
				if(prevMenuItems && prevMenuItems.length > 0){
					dojo.forEach(prevMenuItems, function(menuItem){
     					menu.removeChild(menuItem);
     					menuItem.destroy();
	       			});
				}
			}
		});
	},
	
	showNewRecordDialog: function(typesArray){
		if(!_submitDlg){
			var div = dojo.doc.createElement("div");
			
			// create record type dropdown
			dojo["require"]("dijit.form.FilteringSelect")
			dojo["require"]("dojo.data.ItemFileReadStore");
			var select = new dijit.form.FilteringSelect({
				name: "select",
				store: new dojo.data.ItemFileReadStore({
					data: {
						identifier: "value",
						items: typesArray
					}
				})
			});

			// create label
			var label = dojo.doc.createElement("label");
			label.htmlFor = select.focusNode.id;
			label.innerHTML = messages.RecordType;
			div.appendChild(label);
			div.appendChild(select.domNode);

			// create dialog
			dojo["require"]("com.ibm.rational.cm.web.ui.view.common.ButtonDialog");
			_submitDlg = new com.ibm.rational.cm.web.ui.view.common.ButtonDialog({
				title: messages.CreateNewRecord,
				okLabel: messages.CreateRCD
			});
			_submitDlg.execute = dojo.hitch(this, function(val){
				var value = select.getValue(); 
				if(value){
					dojo.global.location.href = "#action="+cq.ACTION_PREFIX+"newRecord&type=" + encodeURIComponent(value) + "&title=" + encodeURIComponent(select.getDisplayedValue())
				}
			});
			_submitDlg.setContent(div);
			_submitDlg.resize({ w:300, h:100 });
			_submitDlg.startup();
		}
		_submitDlg.show();
	}
});

})();

com.ibm.rational.cm.web.ui.page.submitManager = new com.ibm.rational.cm.web.ui.page.SubmitMenuManager();