/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.client.internal.attributes.PlanItemWorkItemAttribute"); //$NON-NLS-1$

dojo.require("com.ibm.team.foundation.Assert"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.common.Attribute"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.PlanningAttributeType"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.PlanningAttributeValueSet"); //$NON-NLS-1$
// FIXME Layer Breaker
dojo.require("com.ibm.team.apt.web.client.internal.WorkItemAttributeChangeRunnable"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.WorkItemSaveManager"); //$NON-NLS-1$
//dojo.require("com.ibm.team.apt.client.PlanItem"); //$NON-NLS-1$

(function() {

var Assert							= com.ibm.team.foundation.Assert;

var Attribute						= com.ibm.team.workitem.common.Attribute;
var PlanningAttribute				= com.ibm.team.apt.client.PlanningAttribute;
var PlanningAttributeType			= com.ibm.team.apt.client.PlanningAttributeType;
var PlanningAttributeValueSet		= com.ibm.team.apt.client.PlanningAttributeValueSet;
var WorkItemAttributeChangeRunnable	= com.ibm.team.apt.web.client.internal.WorkItemAttributeChangeRunnable;
var WorkItemSaveManager				= com.ibm.team.apt.web.client.internal.WorkItemSaveManager;
var WorkItemProxyFactory			= com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory;

var NULL_MARKER= "_sM_UI_9bEd24fuo2IY3YYw"; //$NON-NLS-1$

dojo.declare("com.ibm.team.apt.client.internal.attributes.PlanItemWorkItemAttribute", PlanningAttribute, { //$NON-NLS-1$
	
	__workItemAttribute: null,
	__valueIndex: 0,

	constructor: function(descriptor, workItemAttribute, valueIndex) {
		this.__addCustomAttributes= "true" == (descriptor.parameters["addCustomAttributes"] || "true").toLowerCase(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ 
		this.__workItemAttribute= workItemAttribute;
		this.__valueIndex= valueIndex;
		this.__valueSet= new PlanningAttributeValueSet(
				workItemAttribute.getValueSet(),
				workItemAttribute.getNullValue());
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	getValue: function(receiver) {
		if (!(receiver instanceof com.ibm.team.apt.client.PlanItem))
			return undefined;
		
		var workItemValue;
		if (receiver.isDraftItem()) {
			workItemValue= receiver.__workItemWorkingCopy.getValue({path: this._getWorkItemWorkingCopyPath()});
		} else {
			workItemValue= receiver.__workItem.attributeValues[this.__valueIndex];
		}
		
		return this._convertToPlanItem(workItemValue, receiver);
	},
	
	setValue: function(receiver, newValue, doSave) {
		if (arguments.length < 3) {
			doSave= true;
		}
		
		if (receiver instanceof com.ibm.team.apt.client.PlanItem) {
			if (this.__addCustomAttributes || this.getValue(receiver) != undefined) {
				receiver.getPlanModel().compoundChange(function(builder) {
					this.__doSetValue(receiver, newValue, doSave);
					receiver._workItemAttributeChanged(this, newValue);
				}, this);
			}
		}
	},
	
	updateFromWorkingCopy: function(receiver, deltaBuilder) {
		Assert.isNotNull(receiver.__workItemWorkingCopy, "WorkItem WorkingCopy is missing"); //$NON-NLS-1$
		
 		var newRawValue= receiver.__workItemWorkingCopy.getValue({path: this._getWorkItemWorkingCopyPath() });
		var newValue= this._convertToPlanItem(newRawValue, receiver);
		var oldValue= this.getValue(receiver);
		
		if (!this.attributeType.equalValues(oldValue, newValue)) {
			this._changed(receiver, oldValue, newValue, deltaBuilder);
			receiver.__workItem.attributeValues[this.__valueIndex]= newRawValue;
		}
 	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------
	
	__doSetValue: function(receiver, newValue, triggerWorkItemSave) {
		var oldValue= this.getValue(receiver);
		if (!this.attributeType.equalValues(oldValue, newValue)) {
			var newRawValue= this._convertToWorkItem(newValue, receiver);
			var newWorkItemValue= newRawValue;
			if (this.attributeType == PlanningAttributeType.DURATION && newRawValue > 0) {
				newWorkItemValue= newWorkItemValue / (1000 * 60 * 60); //work item uses hours instead of milliseconds
			} else if (newRawValue == NULL_MARKER) {
				newWorkItemValue= ""; //$NON-NLS-1$
			}
				
			receiver.getPlanModel().compoundChange(function(builder) {
				this._changed(receiver, oldValue, newValue, builder);
				receiver.__workItem.attributeValues[this.__valueIndex] = newRawValue;

				if (triggerWorkItemSave && !receiver.isDraftItem()) {
					var undoRunnable= dojo.hitch(this, function() {
						this.__doSetValue(receiver, oldValue, false);
					});

					WorkItemSaveManager.save(new WorkItemAttributeChangeRunnable(WorkItemAttributeChangeRunnable.constructorArgs(receiver, this.__workItemAttribute.internalId, newWorkItemValue, undoRunnable)));
				}

			}, this);
			
			var workingCopy= WorkItemProxyFactory.getWorkItemProxy({id: receiver.getId(), createIfNeeded: false});
			if (workingCopy) {
				workingCopy.setValue({path: this._getWorkItemWorkingCopyPath(), attributeId: this.__workItemAttribute.internalId, value: newWorkItemValue});
			}
		}
	},

	/**
	 * Hook to add custom code in subclasses when the attribute value changed. Note that this
	 * is just a workaround for 2.0 M2 -> later on we must allow attributes to listen to other
	 * attributes
	 */
	_changed: function(receiver, oldValue, newValue, deltaBuilder) {
		deltaBuilder.changed(receiver, this, oldValue, newValue);
	},
	
	_convertToPlanItem: function(value, receiver) {
		var result= this.__workItemAttribute.demarshal(value);

		switch (this.__workItemAttribute.attributeType) {
			case Attribute.DATE:
				if (result != null) {
					result= new com.ibm.team.apt.client.internal.Instant(result);
				}
				break;
		}
		
		return result;
	},

	_convertToWorkItem: function(value, receiver) {
		switch (this.__workItemAttribute.attributeType) {
			case Attribute.DATE:
				if (value instanceof com.ibm.team.apt.client.internal.Instant) {
					value= value.getDate();
				}
				break;
		}

		return this.__workItemAttribute.marshal(value);
	},
	
	_getWorkItemWorkingCopyPath: function() {
		return [ "attributes", this.__workItemAttribute.internalId, this._getWorkItemWorkingCopyPathValueId() ]; //$NON-NLS-1$
	},
	
	_getWorkItemWorkingCopyPathValueId: function() {
		switch (this.attributeType) {
			case PlanningAttributeType.STRING:
			case PlanningAttributeType.HTML:
			case PlanningAttributeType.TAGS:
				return "content"; //$NON-NLS-1$
				
			case PlanningAttributeType.INTEGER:
			case PlanningAttributeType.FLOAT:
			case PlanningAttributeType.BOOLEAN:
			case PlanningAttributeType.INSTANT:
				return "id"; //$NON-NLS-1$
				
			case PlanningAttributeType.DURATION:
				return "id"; //$NON-NLS-1$
				
			case PlanningAttributeType.UUID:
			case PlanningAttributeType.ITEM:
			case PlanningAttributeType.ENUMERATION:
			case PlanningAttributeType.WORKITEM_TYPE:
			case PlanningAttributeType.WORKFLOW_STATE:
			default:
				return "id"; //$NON-NLS-1$
		}
	},

	__sentinel: null // terminates this class definition
});

})();
