/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.internal.ui.quickquery.QuickQueryColorizer"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.PlanItem"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.ui.model.ColorizeTag"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.internal.ui.quickquery.QuickQueryParticipant"); //$NON-NLS-1$

(function() {

var ColorizeTag						= com.ibm.team.apt.ui.model.ColorizeTag;
var PlanItem						= com.ibm.team.apt.client.PlanItem;
var QuickQueryParticipant			= com.ibm.team.apt.internal.ui.quickquery.QuickQueryParticipant;

dojo.declare("com.ibm.team.apt.internal.ui.quickquery.QuickQueryColorizer", QuickQueryParticipant, { //$NON-NLS-1$

	__needsRefresh: false,
	__updatesSuppressed: 0,

	__priority: null,
	__color: null,
	__label: null,
	__tag: null,
	
	__runnableToken: null,
	
	constructor: function(viewModel) {
		this._viewModel= viewModel;
		this.__colorizeFn= dojo.hitch(this, this.__doColorize);
	},
	
	// ---- API ------------------------------------------------------------------------------------------------------------

	deferUpdates: function(code, thisObj) {
		this.__updatesSuppressed+= 1;
		try {
			code.call(thisObj);
		} finally {
			this.__updatesSuppressed-= 1;
			if (this.__updatesSuppressed == 0) {
				this._doUnInstall();
			}
		}
	},

	setPriority: function(priority) {
		if (this.__priority != priority) {
			this.__priority= priority;
			this._doUnInstall();
		}
	},

	setLabel: function(label) {
		if (this.__label != label) {
			this.__label= label;
			this._doUnInstall();
		}
	},
	
	setColor: function(color) {
		if (this.__color  && color ? this.__color.toHex() == color.toHex() : this.__color != color) {
			this.__color= color;
			this._doUnInstall();
		}
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------

	_doUnInstall: function() {
		if (this.__updatesSuppressed == 0) {
			this._viewModel.updateModel(function(updateAccessor) {
				var shouldBeInstalled= this._expression != null && this.__color != null && this._enabled;
				this.__needsRefresh= true;

				if (this.__isInstalled ^ shouldBeInstalled) {
					if (shouldBeInstalled) {
						this.__runnableToken= this._viewModel.addPostUpdateRunnable(this.__colorizeFn, true);
						this.__isInstalled= true;
					} else {
						updateAccessor.executeAfterUpdate(dojo.hitch(this, function() {
							this._viewModel.removePostUpdateRunnable(this.__runnableToken);
						}));
						this.__isInstalled= false;
					}
				}

				if (this.__color != null)
					this.__tag= new ColorizeTag(this, this.__color, this.__priority);
				return null;
			}, this);
		};
	},

	__doColorize: function(updateAccessor) {
		updateAccessor.accept(function(entry) {
			var element= entry.getElement();
			if (element instanceof PlanItem) {
				var needsResort= false;
				
				if (this.__isInstalled) {
					if (this.__needsRefresh || updateAccessor.isModified(entry)) {
						updateAccessor.clearTag(entry, this.__tag);
						if (this._expression != null && this._matches(element)) {
							updateAccessor.setTag(entry, this.__tag);
						}
						needsResort= true;
					}
				} else {
					// always clean up - this is our last run before we get uninstalled
					updateAccessor.clearTag(entry, this.__tag);
					needsResort= true;
				}
				
				if (needsResort) {
					updateAccessor.requestChildrenSort(updateAccessor.getParent(entry), false);
				}
			}
			return true;
		}, null, this);
		this.__needsRefresh= false;
	},
	
	__sentinel: null // terminates this class definition
});


})();

